# coding=utf-8

# From django.utils.datetime_safe

# Python's datetime strftime doesn't handle dates before 1900.
# These classes override date and datetime to support the formatting of a date
# through its full "proleptic Gregorian" date range.
#
# Based on code submitted to comp.lang.python by Andrew Dalke
#
# >>> datetime_safe.date(1850, 8, 2).strftime("%Y/%m/%d was a %A")
# '1850/08/02 was a Friday'

from __future__ import unicode_literals
from datetime import date as real_date
from datetime import datetime as real_datetime
import re
import time


class date(real_date):
    def strftime(self, fmt):
        return strftime(self, fmt)


class datetime(real_datetime):
    def strftime(self, fmt):
        return strftime(self, fmt)

    def combine(self, date, time):
        return datetime(date.year, date.month, date.day,
                        time.hour, time.minute, time.microsecond,
                        time.tzinfo)

    def date(self):
        return date(self.year, self.month, self.day)


def new_date(d):
    """Generate a safe date from a datetime.date object."""
    return date(d.year, d.month, d.day)


def new_datetime(d):
    """
    Generate a safe datetime from a datetime.date or datetime.datetime object.
    """
    kw = [d.year, d.month, d.day]
    if isinstance(d, real_datetime):
        kw.extend([d.hour, d.minute, d.second, d.microsecond, d.tzinfo])
    return datetime(*kw)


# This library does not support strftime's "%s" or "%y" format strings.
# Allowed if there's an even number of "%"s because they are escaped.
_illegal_formatting = re.compile(r"((^|[^%])(%%)*%[sy])")


def _findall(text, substr):
    # Also finds overlaps
    sites = []
    i = 0
    while 1:
        j = text.find(substr, i)
        if j == -1:
            break
        sites.append(j)
        i = j + 1
    return sites


def strftime(dt, fmt):
    if dt.year >= 1900:
        return super(type(dt), dt).strftime(fmt)
    illegal_formatting = _illegal_formatting.search(fmt)
    if illegal_formatting:
        msg = 'strftime of dates before 1900 does not handle {0}'
        raise TypeError(msg.format(illegal_formatting.group(0)))

    year = dt.year
    # for every non-leap year century, advance by
    # 6 years to get into the 28-year repeat cycle
    delta = 2000 - year
    off = 6 * (delta // 100 + delta // 400)
    year += off

    # move to around the year 2000
    year += ((2000 - year) // 28) * 28
    timetuple = dt.timetuple()
    s1 = time.strftime(fmt, (year,) + timetuple[1:])
    sites1 = _findall(s1, str(year))

    s2 = time.strftime(fmt, (year + 28,) + timetuple[1:])
    sites2 = _findall(s2, str(year + 28))

    sites = []
    for site in sites1:
        if site in sites2:
            sites.append(site)

    s = s1
    syear = "%04d" % (dt.year,)
    for site in sites:
        s = s[:site] + syear + s[site + 4:]
    return s



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as InternetProvider

class Provider(InternetProvider):

    free_email_domains = (
        'aol.de', 'gmail.com', 'gmx.de', 'googlemail.com', 'hotmail.de',
        'web.de', 'yahoo.de',
    )
    tlds = ('com', 'com', 'com', 'net', 'org', 'de', 'de', 'de', )

    replacements = (
        ('ä', 'ae'), ('Ä', 'Ae'),
        ('ö', 'oe'), ('Ö', 'Oe'),
        ('ü', 'ue'), ('Ü', 'Ue'),
        ('é', 'e'), ('É', 'E'),
        ('à', 'a'), ('À', 'A'),
        ('ß', 'ss'),
    )



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{last_name}} {{company_suffix}}',
        '{{last_name}}',
        '{{last_name}}',
    )

    catch_phrase_formats = (
        '{{catch_phrase_noun}} {{catch_phrase_verb}} {{catch_phrase_attribute}}',
    )

    nouns = (
        'la sécurité', 'le plaisir', 'le confort', 'la simplicité', "l'assurance", "l'art", 'le pouvoir', 'le droit',
        'la possibilité', "l'avantage", 'la liberté'
    )

    verbs = (
        'de rouler', "d'avancer", "d'évoluer", 'de changer', "d'innover", 'de louer', "d'atteindre vos buts",
        'de concrétiser vos projets'
    )

    attributes = (
        'de manière efficace', 'plus rapidement', 'plus facilement', 'plus simplement', 'en toute tranquilité',
        'avant-tout', 'autrement', 'naturellement', 'à la pointe', 'sans soucis', "à l'état pur",
        'à sa source', 'de manière sûre', 'en toute sécurité'
    )

    company_suffixes = ('SA', 'S.A.', 'SARL', 'S.A.R.L.', 'S.A.S.', 'et Fils')

    siren_format = "### ### ###"

    def catch_phrase_noun(self):
        """
        Returns a random catch phrase noun.
        """
        return self.random_element(self.nouns)

    def catch_phrase_attribute(self):
        """
        Returns a random catch phrase attribute.
        """
        return self.random_element(self.attributes)

    def catch_phrase_verb(self):
        """
        Returns a random catch phrase verb.
        """
        return self.random_element(self.verbs)

    def catch_phrase(self):
        """
        :example 'integrate extensible convergence'
        """
        catch_phrase = ""
        while True:

            pattern = self.random_element(self.catch_phrase_formats)
            catch_phrase = self.generator.parse(pattern)
            catch_phrase = catch_phrase[0].upper() + catch_phrase[1:]

            if self._is_catch_phrase_valid(catch_phrase):
                break

        return catch_phrase

    # An array containing string which should not appear twice in a catch phrase
    words_which_should_not_appear_twice = ('sécurité', 'simpl')

    def _is_catch_phrase_valid(self, catch_phrase):
        """
        Validates a french catch phrase.

        :param catch_phrase: The catch phrase to validate.
        """
        for word in self.words_which_should_not_appear_twice:
            # Fastest way to check if a piece of word does not appear twice.
            begin_pos = catch_phrase.find(word)
            end_pos = catch_phrase.find(word, begin_pos + 1)

            if begin_pos != -1 and begin_pos != end_pos: return False

        return True

    def siren(self):
        """
        Generates a siren number (9 digits).
        """
        return self.numerify(self.siren_format)

    def siret(self, max_sequential_digits=2):
        """
        Generates a siret number (14 digits).
        It is in fact the result of the concatenation of a siren number (9 digits),
        a sequential number (4 digits) and a control number (1 digit) concatenation.
        If $max_sequential_digits is invalid, it is set to 2.
        :param max_sequential_digits The maximum number of digits for the sequential number (> 0 && <= 4).
        """
        if max_sequential_digits > 4 or max_sequential_digits <= 0:
            max_sequential_digits = 2

        sequential_number = str(self.random_number(max_sequential_digits)).zfill(4)
        return self.numerify(self.siren() + ' ' + sequential_number + '#')



from __future__ import unicode_literals
from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    # Source: https://en.wikipedia.org/wiki/Telephone_numbers_in_Jordan
    
    cellphone_formats = (
        '+9627{{operator_id}}#######',
        '+962 7 {{operator_id}}### ####',
        '07{{operator_id}}#######',
        '07{{operator_id}} ### ####',
    )

    telephone_formats = (
        '+962{{area_code}}#######',
        '+962 {{area_code}} ### ####',
        '0{{area_code}}#######',
        '0{{area_code}} ### ####',
    )

    services_phones_formats = (
        '9##',
        '12##',
        '13##',
        '14##',
    )

    formats = cellphone_formats + \
              telephone_formats + \
              services_phones_formats

    def operator_id(self):
        return self.random_element([
            '4',
            '7',
            '8',
            '9',
        ])

    def area_code(self):
        return self.random_element([
            '2',
            '3',
            '5',
            '6',
            '7',
        ])

    def cellphone_number(self):
        pattern = self.random_element(self.cellphone_formats)
        return self.numerify(self.generator.parse(pattern))

    def telephone_number(self):
        pattern = self.random_element(self.telephone_formats)
        return self.numerify(self.generator.parse(pattern))

    def service_phone_number(self):
        pattern = self.random_element(self.services_phones_formats)
        return self.numerify(self.generator.parse(pattern))

    def phone_number(self):
        pattern = self.random_element(self.formats)
        return self.numerify(self.generator.parse(pattern))



# coding=utf-8
from __future__ import unicode_literals

from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        'onet.pl',
        'interia.pl',
        'gmail.com',
        'o2.pl',
        'yahoo.com',
        'hotmail.com',
    )

    tlds = ('com', 'com', 'com', 'net', 'org', 'pl', 'pl', 'pl')

    replacements = (
        ('ą', 'a'),
        ('ć', 'c'),
        ('ę', 'e'),
        ('ł', 'l'),
        ('ń', 'n'),
        ('ó', 'o'),
        ('ś', 's'),
        ('ź', 'z'),
        ('ż', 'z'),
    )



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as BaseProvider


# Reference : http://zetawiki.com/wiki/%ED%95%9C%EA%B5%AD%ED%91%9C%EC%A4%80%EC%A7%81%EC%97%85%EB%B6%84%EB%A5%98_%ED%91%9C
# on 2017-08-30
class Provider(BaseProvider):
    jobs=[
        '의회의원/고위공무원 및 공공단체임원',
        '기업고위임원',
        '정부행정 관리자',
        '경영지원 관리자',
        '기타 행정 및 경영지원 관리자',
        '연구 관리자',
        '교육 관리자',
        '법률/경찰/소방 및 교도 관리자',
        '보험 및 금융 관리자',
        '보건의료관련 관리자',
        '사회복지관련 관리자',
        '문화/예술/디자인 및 영상관련 관리자',
        '정보통신관련 관리자',
        '기타 전문서비스 관리자',
        '건설 및 광업 관련 관리자',
        '전기/가스 및 수도 관련 관리자',
        '제품 생산관련 관리자',
        '기타 건설/전기 및 생산 관련 관리자',
        '영업 및 판매 관련 관리자',
        '운송관련 관리자',
        '숙박/여행/오락 및 스포츠 관련 관리자',
        '음식서비스관련 관리자',
        '환경/청소 및 경비 관련 관리자',
        '기타 판매 및 고객 서비스 관리자',
        '생명과학 연구원',
        '자연과학 연구원',
        '인문과학 연구원',
        '사회과학 연구원',
        '생명과학 시험원',
        '농림어업관련 시험원',
        '자연과학 시험원',
        '컴퓨터 하드웨어 기술자 및 연구원',
        '통신공학 기술자 및 연구원',
        '컴퓨터시스템 설계 및 분석가',
        '시스템 소프트웨어 개발자',
        '응용 소프트웨어 개발자',
        '데이터베이스 개발자',
        '네트워크시스템 개발자',
        '컴퓨터 보안 전문가',
        '웹 및 멀티미디어 기획자',
        '웹 개발자',
        '정보 시스템 운영자',
        '통신 및 방송송출 장비 기사',
        '건축가 및 건축공학 기술자',
        '토목공학 기술자',
        '조경 기술자',
        '도시 및 교통설계 전문가',
        '측량 및 지리정보 전문가',
        '건설자재 시험원',
        '화학공학 기술자 및 연구원',
        '화학공학 시험원',
        '금속 / 재료공학 연구원 및 기술자',
        '금속 / 재료공학 시험원',
        '환경공학 기술자 및 연구원',
        '환경공학 시험원',
        '전기공학 기술자 및 연구원',
        '전자공학 기술자 및 연구원',
        '기계공학 기술자 및 연구원',
        '전기/전자 및 기계 공학 시험원',
        '산업안전 및 위험 관리원',
        '보건위생 및 환경 검사원',
        '비파괴 검사원',
        '항공기 조종사',
        '선장/항해사 및 도선사',
        '관제사',
        '식품공학 기술자 및 연구원',
        '섬유공학 기술자 및 연구원',
        '가스/에너지 기술자 및 연구원',
        '소방공학 기술자 및 연구원',
        '식품/섬유 공학 및 에너지 시험원',
        '캐드원',
        '기타 공학관련 기술자 및 시험원',
        '전문 의사',
        '일반 의사',
        '한의사',
        '치과 의사',
        '수의사',
        '약사 및 한약사',
        '간호사',
        '영양사',
        '임상병리사',
        '방사선사',
        '치과기공사',
        '치과위생사',
        '의지보조기기사',
        '물리 및 작업 치료사',
        '임상 심리사 및 기타 치료사',
        '응급구조사',
        '위생사',
        '안경사',
        '의무기록사',
        '간호조무사',
        '안마사',
        '사회복지사',
        '보육 교사',
        '직업상담사 및 취업 알선원',
        '상담 전문가 및 청소년 지도사',
        '시민 단체 활동가',
        '기타 사회복지관련 종사원',
        '성직자',
        '기타 종교관련 종사자',
        '대학 교수',
        '대학 시간강사',
        '중/고등학교 교사',
        '초등학교 교사',
        '특수교육 교사',
        '유치원 교사',
        '문리 및 어학 강사',
        '컴퓨터 강사',
        '기술 및 기능계 강사',
        '예능 강사',
        '학습지 및 방문 교사',
        '기타 문리/기술 및 예능 강사',
        '장학관/연구관 및 교육 관련 전문가',
        '대학 교육조교',
        '보조 교사 및 기타 교사',
        '판사 및 검사',
        '변호사',
        '법무사 및 집행관',
        '변리사',
        '정부 및 공공 행정 전문가',
        '인사 및 노사 관련 전문가',
        '회계사',
        '세무사',
        '관세사',
        '경영 및 진단 전문가',
        '투자 및 신용 분석가',
        '자산 운용가',
        '보험 및 금융 상품 개발자',
        '증권 및 외환 딜러',
        '손해사정인',
        '기타 금융 및 보험 관련 전문가',
        '상품기획 전문가',
        '여행상품 개발자',
        '광고 및 홍보 전문가',
        '조사 전문가',
        '행사기획자',
        '감정평가 전문가',
        '해외 영업원',
        '기술 영업원',
        '상품중개인 및 경매사',
        '부동산 컨설턴트 및 중개인',
        '기타 기술영업 및 중개 관련 종사자',
        '작가 및 관련 전문가',
        '번역가',
        '통역가',
        '기자 및 논설위원',
        '출판물 전문가',
        '큐레이터 및 문화재 보존원',
        '사서 및 기록물관리사',
        '감독 및 기술감독',
        '배우 및 모델',
        '아나운서 및 리포터',
        '촬영기사',
        '음향 및 녹음 기사',
        '영상/녹화 및 편집 기사',
        '조명기사 및 영사기사',
        '기타 연극/영화 및 영상 관련 종사자',
        '화가 및 조각가',
        '사진기자 및 사진가',
        '만화가 및 만화영화 작가',
        '국악 및 전통예능인',
        '지휘자/작곡가 및 연주가',
        '가수 및 성악가',
        '무용가 및 안무가',
        '제품 디자이너',
        '패션 디자이너',
        '실내장식 디자이너',
        '시각 디자이너',
        '웹 및 멀티미디어 디자이너',
        '경기감독 및 코치',
        '직업 운동선수',
        '경기심판 및 경기기록원',
        '스포츠 및 레크레이션 강사',
        '기타 스포츠 및 레크레이션 관련 전문가',
        '연예인 및 스포츠 매니저',
        '마술사 및 기타 문화/ 예술 관련 종사자',
        '조세행정 사무원',
        '관세행정 사무원',
        '병무행정 사무원',
        '국가/지방 및 공공행정 사무원',
        '기획 및 마케팅 사무원',
        '인사 및 교육/훈련 사무원',
        '자재관리 사무원',
        '생산 및 품질 관리 사무원',
        '무역 사무원',
        '운송 사무원',
        '총무 사무원',
        '회계 사무원',
        '경리 사무원',
        '비서',
        '전산 자료 입력원 및 사무 보조원',
        '출납창구 사무원',
        '보험 심사원 및 사무원',
        '금융관련 사무원',
        '신용 추심원',
        '법률관련 사무원',
        '감사 사무원',
        '통계관련 사무원',
        '여행 사무원',
        '안내 / 접수 사무원 및 전화교환원',
        '고객 상담 및 모니터 요원',
        '기타 사무원',
        '경찰관',
        '소방관',
        '소년보호관 및 교도관',
        '경호원',
        '청원 경찰',
        '무인 경비원',
        '기타 경호 및 보안 관련 종사원',
        '간병인',
        '기타 의료/복지 관련 서비스 종사원',
        '이용사',
        '미용사',
        '피부미용 및 체형관리사',
        '메이크업 아티스트 및 분장사',
        '애완동물 미용사',
        '기타 미용관련 서비스 종사원',
        '결혼 상담원 및 웨딩플래너',
        '혼례 종사원',
        '장례 상담원 및 장례 지도사',
        '기타 이미용/예식 및 의료보조 서비스 종사원',
        '항공기 객실승무원',
        '선박 및 열차 객실승무원',
        '여행 및 관광통역 안내원',
        '숙박시설 서비스원',
        '오락시설 서비스원',
        '기타 여가 및 스포츠 관련 종사원',
        '한식 주방장 및 조리사',
        '중식 주방장 및 조리사',
        '양식 주방장 및 조리사',
        '일식 주방장 및 조리사',
        '기타 주방장 및 조리사',
        '바텐더',
        '웨이터',
        '기타 음식서비스 종사원',
        '자동차 영업원',
        '제품 및 광고 영업원',
        '보험 설계사 및 간접투자증권 판매인',
        '상점 판매원',
        '매표원 및 복권 판매원',
        '매장계산원 및 요금정산원',
        '상품 대여원',
        '방문 판매원',
        '통신서비스판매원',
        '텔레마케터',
        '인터넷 판매원',
        '노점 및 이동 판매원',
        '홍보 도우미 및 판촉원',
        '곡식작물 재배원',
        '채소 및 특용작물 재배원',
        '과수작물 재배원',
        '원예작물 재배원',
        '조경원',
        '낙농업관련 종사원',
        '가축 사육 종사원',
        '기타 사육관련 종사원',
        '조림/영림 및 벌목원',
        '임산물채취 및 기타 임업 관련 종사원',
        '양식원',
        '어부 및 해녀',
        '제빵원 및 제과원',
        '떡제조원',
        '정육원 및 도축원',
        '식품 및 담배 등급원',
        '김치 및 밑반찬 제조 종사원',
        '기타 식품가공관련 종사원',
        '패턴사',
        '재단사',
        '재봉사',
        '제화원',
        '기타 섬유 및 가죽 관련 기능 종사원',
        '한복 제조원',
        '양장 및 양복 제조원',
        '모피 및 가죽의복 제조원',
        '의복/가죽 및 모피 수선원',
        '기타 의복 제조원',
        '목제품 제조관련 종사원',
        '가구 제조 및 수리원',
        '악기제조 및 조율사',
        '간판 제작 및 설치원',
        '금형원',
        '주조원',
        '단조원',
        '제관원',
        '판금원',
        '용접원',
        '자동차 정비원',
        '항공기 정비원',
        '선박 정비원',
        '철도 기관차 및 전동차 정비원',
        '기타 운송장비 정비원',
        '공업기계 설치 및 정비원',
        '승강기 설치 및 정비원',
        '물품 이동 장비 설치 및 정비원',
        '냉동/냉장 /공조기 설치 및 정비원',
        '보일러 설치 및 정비원',
        '건설 및 광업기계 설치 및 정비원',
        '농업용 및 기타 기계장비 설치 및 정비원',
        '',
        '가전제품 설치 및 수리원',
        '기타 전기/전자기기 설치 및 수리원',
        '산업전공',
        '내선전공',
        '외선전공',
        '강구조물 가공원 및 건립원',
        '경량 철골공',
        '철근공',
        '콘크리트공',
        '건축 석공',
        '건축 목공',
        '조적공 및 석재 부설원',
        '기타 건설관련 기능 종사원',
        '미장공',
        '방수공',
        '단열공',
        '바닥재 시공원',
        '도배공 및 유리 부착원',
        '건축 도장공',
        '섀시 조립 및 설치원',
        '기타 건축마감관련 기능 종사원',
        '광원/채석원 및 석재 절단원',
        '철로 설치 및 보수원',
        '기타 채굴 및 토목 관련 종사자',
        '영상 및 관련 장비 설치 및 수리원',
        '통신 및 관련 장비 설치 및 수리원',
        '통신/방송 및 인터넷 케이블 설치 및 수리원',
        '공예원',
        '귀금속 및 보석 세공원',
        '건설 배관공',
        '공업 배관공',
        '기타 배관공',
        '배관 세정원 및 방역원',
        '기타 기능관련 종사원',
        '제분 및 도정 관련 기계 조작원',
        '곡물가공제품 기계 조작원',
        '육류/어패류 및 낙농품 가공 기계조작원',
        '과실 및 채소 관련 기계조작원',
        '음료 제조관련 기계 조작원',
        '기타 식품가공관련 기계조작원',
        '섬유제조 기계조작원',
        '표백 및 염색 관련 조작원',
        '직조기 및 편직기 조작원',
        '신발제조기 조작원 및 조립원',
        '기타 직물 및 신발 관련 기계조작원 및 조립원',
        '세탁관련 기계조작원',
        '석유 및 천연가스제조 관련 제어장치 조작원',
        '화학물 가공장치 조작원',
        '기타 석유 및 화학물 가공장치 조작원',
        '화학제품 생산기 조작원',
        '타이어 및 고무제품 생산기 조작원',
        '플라스틱제품 생산기 조작원',
        '고무 및 플라스틱 제품 조립원',
        '주조기 조작원',
        '단조기 조작원',
        '용접기 조작원',
        '금속가공관련 제어장치 조작원',
        '금속가공 기계조작원',
        '제관기 조작원',
        '판금기 조작원',
        '도장기 조작원',
        '도금 및 금속분무기 조작원',
        '유리제조 및 가공기 조작원',
        '점토제품 생산기 조작원',
        '시멘트 및 광물제품 제조기 조작원',
        '광석 및 석제품 가공기 조작원',
        '기타 비금속제품관련 생산기 조작원',
        '금속공작기계 조작원',
        '냉/난방 관련 설비 조작원',
        '자동조립라인 및 산업용 로봇 조작원',
        '자동차 조립원',
        '자동차 부분품 조립원',
        '운송장비 조립원',
        '일반기계 조립원',
        '금속기계부품 조립원',
        '발전 및 배전장치 조작원',
        '전기 및 전자 설비 조작원',
        '전기 부품 및 제품제조 기계조작원',
        '전자 부품 및 제품 제조 기계조작원',
        '전기/전자 부품 및 제품 조립원',
        '철도 및 전동차 기관사',
        '화물열차 차장 및 관련 종사원',
        '택시 운전원',
        '버스 운전원',
        '화물차 및 특수차 운전원',
        '기타 자동차 운전원',
        '물품이동 장비 조작원',
        '건설 및 채굴 기계 운전원',
        '선박 갑판승무원 및 관련 종사원',
        '상/하수도 처리장치 조작원',
        '재활용 처리 및 소각로 조작원',
        '목재 가공관련 기계 조작원',
        '가구조립원',
        '펄프 및 종이 제조장치 조작원',
        '종이제품 생산기 조작원',
        '기타 목재 및 종이 관련 기계조작원',
        '인쇄기 조작원',
        '사진인화 및 현상기 조작원',
        '기타 제조관련 기계 조작원',
        '건설 및 광업 단순 종사원',
        '하역 및 적재 단순 종사원',
        '우편물 집배원',
        '택배원',
        '음식 배달원',
        '기타 배달원',
        '제조관련 단순 종사원',
        '청소원',
        '환경 미화원 및 재활용품 수거원',
        '경비원',
        '검표원',
        '가사 도우미',
        '육아 도우미',
        '패스트푸드원',
        '주방 보조원',
        '주유원',
        '기타 판매관련 단순 종사원',
        '농림어업관련 단순 종사원',
        '계기 검침원 및 가스점검원',
        '수금원',
        '주차 관리원 및 안내원',
        '구두 미화원',
        '세탁원 및 다림질원',
        '기타 서비스관련 단순 종사원',
        '영관급 이상',
        '위관급',
        '장기 부사관 및 준위'
    ]



from __future__ import unicode_literals
from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    formats = (
        '+34 ### ### ###',
        '+34 #########',
        '+34 ### ## ## ##',
        '+34### ### ###',
        '+34#########',
        '+34### ## ## ##',
    )


localized = True

# 'Latin' is the default locale
default_locale = 'la'


from .. import BaseProvider


class Provider(BaseProvider):
    """Will provide methods to retrieve lorem content

    Attributes:
        sentence_punctuation (str): End of sentence punctuation
        word_connector (str): Default connector between words

    Methods:
        word: Generate a random word
        words: Generate a list containing random words
        sentence: Generate a random sentence
        sentences: Generate a list containing sentences
        paragraph: Generate a single paragraph
        paragraphs: Generate many paragraphs
        text: Generate a text string.
    """
    word_connector = ' '
    sentence_punctuation = '.'

    def word(self, ext_word_list=None):
        """
        :returns: A random word, eg: 'lorem'

        :param ext_word_list: a list of words you would like to have instead of
            'Lorem ipsum'

        :rtype: str
        """
        if ext_word_list:
            return self.random_element(ext_word_list)
        return self.random_element(self.word_list)

    def words(self, nb=3, ext_word_list=None):
        """
        :returns: An array of random words. for example: ['Lorem', 'ipsum', 'dolor']

        Keyword arguments:
        :param nb: how many words to return
        :param ext_word_list: a list of words you would like to have instead of
            'Lorem ipsum'

        :rtype: list
        """
        return [self.word(ext_word_list) for _ in range(0, nb)]

    def sentence(self, nb_words=6, variable_nb_words=True, ext_word_list=None):
        """
        Generate a random sentence
        :example 'Lorem ipsum dolor sit amet.'

        :param nb_words: around how many words the sentence should contain
        :param variable_nb_words: set to false if you want exactly ``nb``
            words returned, otherwise the result may include a number of words
            of ``nb`` +/-40% (with a minimum of 1)
        :param ext_word_list: a list of words you would like to have instead of
            'Lorem ipsum'.

        :rtype: str
        """
        if nb_words <= 0:
            return ''

        if variable_nb_words:
            nb_words = self.randomize_nb_elements(nb_words, min=1)

        words = self.words(nb=nb_words, ext_word_list=ext_word_list)
        words[0] = words[0].title()

        return self.word_connector.join(words) + self.sentence_punctuation

    def sentences(self, nb=3, ext_word_list=None):
        """
        Generate an array of sentences
        :example ['Lorem ipsum dolor sit amet.', 'Consectetur adipisicing eli.']

        Keyword arguments:
        :param nb: how many sentences to return
        :param ext_word_list: a list of words you would like to have instead of
            'Lorem ipsum'.

        :rtype: list
        """
        return [self.sentence(ext_word_list=ext_word_list) for _ in range(0, nb)]

    def paragraph(self, nb_sentences=3, variable_nb_sentences=True, ext_word_list=None):
        """
        :returns: A single paragraph. For example: 'Sapiente sunt omnis. Ut
            pariatur ad autem ducimus et. Voluptas rem voluptas sint modi dolorem amet.'

        Keyword arguments:
        :param nb_sentences: around how many sentences the paragraph should contain
        :param variable_nb_sentences: set to false if you want exactly ``nb``
            sentences returned, otherwise the result may include a number of
            sentences of ``nb`` +/-40% (with a minimum of 1)
        :param ext_word_list: a list of words you would like to have instead of
            'Lorem ipsum'.

        :rtype: str
        """
        if nb_sentences <= 0:
            return ''

        if variable_nb_sentences:
            nb_sentences = self.randomize_nb_elements(nb_sentences, min=1)

        para = self.word_connector.join(self.sentences(
            nb_sentences, ext_word_list=ext_word_list
        ))

        return para

    def paragraphs(self, nb=3, ext_word_list=None):
        """
        Generate an array of paragraphs
        :example [paragraph1, paragraph2, paragraph3]
        :param nb: how many paragraphs to return
        :param ext_word_list: a list of words you would like to have instead of
            'Lorem ipsum'.

        :rtype: list
        """

        return [self.paragraph(ext_word_list=ext_word_list) for _ in range(0, nb)]

    def text(self, max_nb_chars=200, ext_word_list=None):
        """
        Generate a text string.
        Depending on the ``max_nb_chars, returns a string made of words, sentences, or paragraphs.
        :example 'Sapiente sunt omnis. Ut pariatur ad autem ducimus et. Voluptas rem voluptas sint modi dolorem amet.'

        Keyword arguments:
        :param max_nb_chars: Maximum number of characters the text should contain (minimum 5)
        :param ext_word_list: a list of words you would like to have instead of 'Lorem ipsum'.

        :rtype str
        """
        text = []
        if max_nb_chars < 5:
            raise ValueError('text() can only generate text of at least 5 characters')

        if max_nb_chars < 25:
            # join words
            while not text:
                size = 0
                # determine how many words are needed to reach the $max_nb_chars once;
                while size < max_nb_chars:
                    word = (self.word_connector if size else '') + self.word(ext_word_list=ext_word_list)
                    text.append(word)
                    size += len(word)
                text.pop()
            text[0] = text[0][0].upper() + text[0][1:]
            last_index = len(text) - 1
            text[last_index] += self.sentence_punctuation
        elif max_nb_chars < 100:
            # join sentences
            while not text:
                size = 0
                # determine how many sentences are needed to reach the $max_nb_chars once
                while size < max_nb_chars:
                    sentence = (self.word_connector if size else '') + self.sentence(ext_word_list=ext_word_list)
                    text.append(sentence)
                    size += len(sentence)
                text.pop()
        else:
            # join paragraphs
            while not text:
                size = 0
                # determine how many paragraphs are needed to reach the $max_nb_chars once
                while size < max_nb_chars:
                    paragraph = ('\n' if size else '') + self.paragraph(ext_word_list=ext_word_list)
                    text.append(paragraph)
                    size += len(paragraph)
                text.pop()

        return "".join(text)



from __future__ import unicode_literals
from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        '{{company_prefix}} {{last_name}}',
        '{{company_prefix}} {{last_name}} {{last_name}}',
        '{{company_prefix}} {{last_name}} {{company_suffix}}',
        '{{company_prefix}} {{last_name}} {{last_name}} {{company_suffix}}',
    )

    # From http://id.wikipedia.org/wiki/Jenis_badan_usaha
    # via https://github.com/fzaninotto/faker/blob/master/src/Faker/Provider/id_ID/Company.php
    company_prefixes = (
        'PT', 'CV', 'UD', 'PD', 'Perum',
    )

    # From http://id.wikipedia.org/wiki/Jenis_badan_usaha
    # via https://github.com/fzaninotto/faker/blob/master/src/Faker/Provider/id_ID/Company.php
    company_suffixes = (
        '(Persero) Tbk', 'Tbk',
    )

    def company_prefix(self):
        return self.random_element(self.company_prefixes)



# coding=utf-8
from __future__ import unicode_literals

from text_unidecode import unidecode

from .. import BaseProvider

from ipaddress import ip_address, ip_network, IPV4LENGTH, IPV6LENGTH

# from faker.generator import random
# from faker.providers.lorem.la import Provider as Lorem
from faker.utils.decorators import lowercase, slugify, slugify_unicode


localized = True


class Provider(BaseProvider):
    safe_email_tlds = ('org', 'com', 'net')
    free_email_domains = ('gmail.com', 'yahoo.com', 'hotmail.com')
    tlds = (
        'com', 'com', 'com', 'com', 'com', 'com', 'biz', 'info', 'net', 'org'
    )

    uri_pages = (
        'index', 'home', 'search', 'main', 'post', 'homepage', 'category',
        'register', 'login', 'faq', 'about', 'terms', 'privacy', 'author'
    )
    uri_paths = (
        'app', 'main', 'wp-content', 'search', 'category', 'tag', 'categories',
        'tags', 'blog', 'posts', 'list', 'explore'
    )
    uri_extensions = (
        '.html', '.html', '.html', '.htm', '.htm', '.php', '.php', '.jsp',
        '.asp'
    )

    user_name_formats = (
        '{{last_name}}.{{first_name}}',
        '{{first_name}}.{{last_name}}',
        '{{first_name}}##',
        '?{{last_name}}',
    )
    email_formats = (
        '{{user_name}}@{{domain_name}}',
        '{{user_name}}@{{free_email_domain}}',
    )
    url_formats = (
        'http://www.{{domain_name}}/',
        'http://{{domain_name}}/',
        'https://www.{{domain_name}}/',
        'https://{{domain_name}}/',
    )
    uri_formats = (
        '{{url}}',
        '{{url}}{{uri_page}}/',
        '{{url}}{{uri_page}}{{uri_extension}}',
        '{{url}}{{uri_path}}/{{uri_page}}/',
        '{{url}}{{uri_path}}/{{uri_page}}{{uri_extension}}',
    )
    image_placeholder_services = (
        'https://placeholdit.imgix.net/~text'
        '?txtsize=55&txt={width}x{height}&w={width}&h={height}',
        'https://www.lorempixel.com/{width}/{height}',
        'https://dummyimage.com/{width}x{height}',
    )

    replacements = tuple()

    def _to_ascii(self, string):
        for search, replace in self.replacements:
            string = string.replace(search, replace)

        string = unidecode(string)
        return string

    @lowercase
    def email(self):
        pattern = self.random_element(self.email_formats)
        return "".join(self.generator.parse(pattern).split(" "))

    @lowercase
    def safe_email(self):
        return '{}@example.{}'.format(
            self.user_name(), self.random_element(self.safe_email_tlds)
        )

    @lowercase
    def free_email(self):
        return self.user_name() + '@' + self.free_email_domain()

    @lowercase
    def company_email(self):
        return self.user_name() + '@' + self.domain_name()

    @lowercase
    def free_email_domain(self):
        return self.random_element(self.free_email_domains)

    @lowercase
    def ascii_email(self):
        pattern = self.random_element(self.email_formats)
        return self._to_ascii(
            "".join(self.generator.parse(pattern).split(" "))
        )

    @lowercase
    def ascii_safe_email(self):
        return self._to_ascii(
            self.user_name() +
            '@example.' +
            self.random_element(self.safe_email_tlds)
        )

    @lowercase
    def ascii_free_email(self):
        return self._to_ascii(
            self.user_name() + '@' + self.free_email_domain()
        )

    @lowercase
    def ascii_company_email(self):
        return self._to_ascii(
            self.user_name() + '@' + self.domain_name()
        )

    @slugify_unicode
    def user_name(self):
        pattern = self.random_element(self.user_name_formats)
        username = self._to_ascii(
            self.bothify(self.generator.parse(pattern)).lower()
        )
        return username

    @lowercase
    def domain_name(self, levels=1):
        """
        Produce an Internet domain name with the specified number of
        subdomain levels.

        >>> domain_name()
        nichols-phillips.com
        >>> domain_name(2)
        williamson-hopkins.jackson.com
        """
        if levels < 1:
            raise ValueError("levels must be greater than or equal to 1")
        if levels == 1:
            return self.domain_word() + '.' + self.tld()
        else:
            return self.domain_word() + '.' + self.domain_name(levels - 1)

    @lowercase
    @slugify_unicode
    def domain_word(self,):
        company = self.generator.format('company')
        company_elements = company.split(' ')
        company = self._to_ascii(company_elements.pop(0))
        return company

    def tld(self):
        return self.random_element(self.tlds)

    def url(self):
        pattern = self.random_element(self.url_formats)
        return self.generator.parse(pattern)

    def ipv4(self, network=False):
        """Produce a random IPv4 address or network with a valid CIDR"""
        address = str(ip_address(self.generator.random.randint(
            0, (2 ** IPV4LENGTH) - 1)))
        if network:
            address += '/' + str(self.generator.random.randint(0, IPV4LENGTH))
            address = str(ip_network(address, strict=False))
        return address

    def ipv6(self, network=False):
        """Produce a random IPv6 address or network with a valid CIDR"""
        address = str(ip_address(self.generator.random.randint(
            2 ** IPV4LENGTH, (2 ** IPV6LENGTH) - 1)))
        if network:
            address += '/' + str(self.generator.random.randint(0, IPV6LENGTH))
            address = str(ip_network(address, strict=False))
        return address

    def mac_address(self):
        mac = [self.generator.random.randint(0x00, 0xff) for i in range(0, 6)]
        return ":".join(map(lambda x: "%02x" % x, mac))

    def uri_page(self):
        return self.random_element(self.uri_pages)

    def uri_path(self, deep=None):
        deep = deep if deep else self.generator.random.randint(1, 3)
        return "/".join(
            [self.random_element(self.uri_paths) for _ in range(0, deep)]
        )

    def uri_extension(self):
        return self.random_element(self.uri_extensions)

    def uri(self):
        pattern = self.random_element(self.uri_formats)
        return self.generator.parse(pattern)

    @slugify
    def slug(self, value=None):
        """Django algorithm"""
        if value is None:
            value = self.generator.text(20)
        return value

    def image_url(self, width=None, height=None):
        """
        Returns URL to placeholder image
        Example: http://placehold.it/640x480
        """
        width_ = width or self.random_int(max=1024)
        height_ = height or self.random_int(max=1024)
        placeholder_url = self.random_element(self.image_placeholder_services)
        return placeholder_url.format(width=width_, height=height_)



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{last_name}} {{company_suffix}}',
        '{{last_name}}'
    )

    company_suffixes = (
        'Kft',
        'és Tsa',
        'Kht',
        'ZRT',
        'NyRT',
        'BT'
    )

    def company_suffix(self):
        return self.random_element(self.company_suffixes)



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as InternetProvider


class Provider(InternetProvider):
    safe_email_tlds = ('com', 'net', 'fr', 'fr')
    free_email_domains = (
    'voila.fr', 'gmail.com', 'hotmail.fr', 'yahoo.fr', 'laposte.net', 'free.fr', 'sfr.fr', 'orange.fr', 'bouygtel.fr',
    'club-internet.fr', 'dbmail.com', 'live.com', 'ifrance.com', 'noos.fr', 'tele2.fr', 'tiscali.fr', 'wanadoo.fr')
    tlds = ('com', 'com', 'com', 'net', 'org', 'fr', 'fr', 'fr')
    replacements = (
        ('à', 'a'), ('â', 'a'), ('ä', 'a'),
        ('ç', 'c'),
        ('é', 'e'), ('è', 'e'), ('ê', 'e'), ('ë', 'e'), ('É', 'e'),
        ('ï', 'i'), ('î', 'i'),
        ('ô', 'o'), ('ö', 'o'),
        ('ù', 'u'), ('ü', 'u'),
    )



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as AddressProvider

class Provider(AddressProvider):
    building_number_formats = ('#', '##', '###', '#', '##', '###',)

    street_suffixes = (
        'baan', 'boulevard', 'dreef', 'hof', 'laan', 'lei', 'pad',
        'ring', 'singel', 'steeg', 'straat', 'weg',
    )

    # the 4 digit numerical part of Belgium postal codes is between 1000 and 9999;
    # see https://nl.wikipedia.org/wiki/Postcode#Postnummers_in_België
    postcode_formats = ('%###',)

    city_formats = ('{{city}}',)

    # countries are from http://nl.wikipedia.org/wiki/ISO_3166-1
    countries = (
        'Afghanistan', 'Albanië', 'Algerije', 'Amerikaans-Samoa',
        'Amerikaanse Maagdeneilanden', 'Andorra', 'Angola', 'Anguilla',
        'Antarctica', 'Antigua en Barbuda', 'Argentinië', 'Armenië', 'Aruba',
        'Australië', 'Azerbeidzjan', "Bahama's", 'Bahrein', 'Bangladesh',
        'Barbados', 'België', 'Belize', 'Benin', 'Bermuda', 'Bhutan',
        'Bolivia', 'Bonaire, Sint Eustatius en Saba', 'Bosnië en Herzegovina',
        'Botswana', 'Bouveteiland', 'Brazilië',
        'Brits Indische Oceaanterritorium', 'Britse Maagdeneilanden', 'Brunei',
        'Bulgarije', 'Burkina Faso', 'Burundi', 'Cambodja', 'Canada',
        'Centraal-Afrikaanse Republiek', 'Chili', 'China', 'Christmaseiland',
        'Cocoseilanden', 'Colombia', 'Comoren', 'Congo-Brazzaville',
        'Congo-Kinshasa', 'Cookeilanden', 'Costa Rica', 'Cuba', 'Curaçao',
        'Cyprus', 'Denemarken', 'Djibouti', 'Dominica',
        'Dominicaanse Republiek', 'Duitsland', 'Ecuador', 'Egypte',
        'El Salvador', 'Equatoriaal-Guinea', 'Eritrea', 'Estland', 'Ethiopië',
        'Faeröer', 'Falklandeilanden', 'Fiji', 'Filipijnen', 'Finland',
        'Frankrijk', 'Frans-Guyana', 'Frans-Polynesië',
        'Franse Zuidelijke en Antarctische Gebieden', 'Gabon', 'Gambia',
        'Georgië', 'Ghana', 'Gibraltar', 'Grenada', 'Griekenland', 'Groenland',
        'Guadeloupe', 'Guam', 'Guatemala', 'Guernsey', 'Guinee',
        'Guinee-Bissau', 'Guyana', 'Haïti', 'Heard en McDonaldeilanden',
        'Honduras', 'Hongarije', 'Hongkong', 'IJsland', 'Ierland', 'India',
        'Indonesië', 'Irak', 'Iran', 'Israël', 'Italië', 'Ivoorkust',
        'Jamaica', 'Japan', 'Jemen', 'Jersey', 'Jordanië', 'Kaaimaneilanden',
        'Kaapverdië', 'Kameroen', 'Kazachstan', 'Kenia', 'Kirgizië',
        'Kiribati', 'Kleine Pacifische eilanden van de Verenigde Staten',
        'Koeweit', 'Kroatië', 'Laos', 'Lesotho', 'Letland', 'Libanon',
        'Liberia', 'Libië', 'Liechtenstein', 'Litouwen', 'Luxemburg', 'Macau',
        'Macedonië', 'Madagaskar', 'Malawi', 'Maldiven', 'Maleisië', 'Mali',
        'Malta', 'Man', 'Marokko', 'Marshalleilanden', 'Martinique',
        'Mauritanië', 'Mauritius', 'Mayotte', 'Mexico', 'Micronesia',
        'Moldavië', 'Monaco', 'Mongolië', 'Montenegro', 'Montserrat',
        'Mozambique', 'Myanmar', 'Namibië', 'Nauru', 'Nederland', 'Nepal',
        'Nicaragua', 'Nieuw-Caledonië', 'Nieuw-Zeeland', 'Niger', 'Nigeria',
        'Niue', 'Noord-Korea', 'Noordelijke Marianen', 'Noorwegen', 'Norfolk',
        'Oeganda', 'Oekraïne', 'Oezbekistan', 'Oman', 'Oost-Timor',
        'Oostenrijk', 'Pakistan', 'Palau', 'Palestina', 'Panama',
        'Papoea-Nieuw-Guinea', 'Paraguay', 'Peru', 'Pitcairneilanden', 'Polen',
        'Portugal', 'Puerto Rico', 'Qatar', 'Roemenië', 'Rusland', 'Rwanda',
        'Réunion', 'Saint Kitts en Nevis', 'Saint Lucia',
        'Saint Vincent en de Grenadines', 'Saint-Barthélemy',
        'Saint-Pierre en Miquelon', 'Salomonseilanden', 'Samoa', 'San Marino',
        'Sao Tomé en Principe', 'Saoedi-Arabië', 'Senegal', 'Servië',
        'Seychellen', 'Sierra Leone', 'Singapore', 'Sint Maarten',
        'Sint-Helena, Ascension en Tristan da Cunha', 'Sint-Maarten',
        'Slovenië', 'Slowakije', 'Soedan', 'Somalië', 'Spanje',
        'Spitsbergen en Jan Mayen', 'Sri Lanka', 'Suriname', 'Swaziland',
        'Syrië', 'Tadzjikistan', 'Taiwan', 'Tanzania', 'Thailand', 'Togo',
        'Tokelau', 'Tonga', 'Trinidad en Tobago', 'Tsjaad', 'Tsjechië',
        'Tunesië', 'Turkije', 'Turkmenistan', 'Turks- en Caicoseilanden',
        'Tuvalu', 'Uruguay', 'Vanuatu', 'Vaticaanstad', 'Venezuela',
        'Verenigd Koninkrijk', 'Verenigde Arabische Emiraten',
        'Verenigde Staten', 'Vietnam', 'Wallis en Futuna', 'Westelijke Sahara',
        'Wit-Rusland', 'Zambia', 'Zimbabwe', 'Zuid-Afrika',
        'Zuid-Georgia en de Zuidelijke Sandwicheilanden', 'Zuid-Korea',
        'Zuid-Soedan', 'Zweden', 'Zwitserland', 'Åland',
    )

    # cities as listed on "postcodezoeker"
    # http://www.postcodes-maps.be/postcodelijst.php
    cities = (
		"'s Herenelderen","'s-Gravenvoeren","'s-Gravenwezel","Aaigem",
		"Aalbeke","Aalst","Aalter","Aarschot",
		"Aarsele","Aartrijke","Aartselaar","Abolens",
		"Abée","Achel","Achet","Achêne",
		"Acosse","Acoz","Adegem","Adinkerke",
		"Affligem","Afsnee","Agimont","Aineffe",
		"Aische-en-Refail","Aiseau","Aiseau-Presles","Aisemont",
		"Alken","Alle","Alleur","Alsemberg",
		"Alveringem","Amay","Amberloup","Ambly",
		"Ambresin","Amel","Amonines","Amougies",
		"Ampsin","Andenne","Anderlecht","Anderlues",
		"Andrimont","Angleur","Angre","Angreau",
		"Anhée","Anlier","Anloy","Annevoie-Rouillon",
		"Ans","Anseremme","Anseroeul","Antheit",
		"Anthisnes","Anthée","Antoing","Antwerpen",
		"Anvaing","Anzegem","Appels","Appelterre-Eichem",
		"Arbre","Arbrefontaine","Arc-Ainières","Arc-Wattripont",
		"Archennes","Ardooie","Arendonk","Argenteau",
		"Arlon","Arquennes","Arsimont","Arville",
		"As","Aspelare","Asper","Asquillies",
		"Asse","Assebroek","Assenede","Assenois",
		"Assent","Assesse","Astene","Ath",
		"Athis","Athus","Attenhoven","Attenrode",
		"Attert","Attre","Aubange","Aubechies",
		"Aubel","Aublain","Auby-sur-Semois","Audregnies",
		"Aulnois","Autelbas","Autre-Eglise","Autreppe",
		"Auvelais","Ave-et-Auffe","Avekapelle","Avelgem",
		"Avennes","Averbode","Avernas-le-Bauduin","Avin",
		"Awans","Awenne","Awirs","Aye",
		"Ayeneux","Aywaille","Baaigem","Baal",
		"Baardegem","Baarle-Hertog","Baasrode","Bachte-Maria-Leerne",
		"Baelen","Bagimont","Baileux","Bailièvre",
		"Baillamont","Bailleul","Baillonville","Baisieux",
		"Baisy-Thy","Balegem","Balen","Balâtre",
		"Bambrugge","Bande","Barbençon","Barchon",
		"Baronville","Barry","Barvaux-Condroz","Barvaux-sur-Ourthe",
		"Bas-Oha","Basse-Bodeux","Bassenge","Bassevelde",
		"Bassilly","Bastogne","Basècles","Batsheers",
		"Battice","Battignies","Baudour","Bauffe",
		"Baugnies","Baulers","Bavegem","Bavikhove",
		"Bazel","Beaufays","Beaumont","Beauraing",
		"Beausaint","Beauvoorde","Beauwelz","Beclers",
		"Beek","Beerlegem","Beernem","Beerse",
		"Beersel","Beerst","Beert","Beervelde",
		"Beerzel","Beez","Beffe","Begijnendijk",
		"Beho","Beigem","Bekegem","Bekkerzeel",
		"Bekkevoort","Belgrade","Bellaire","Bellecourt",
		"Bellefontaine","Bellegem","Bellem","Bellevaux",
		"Bellevaux-Ligneuville","Bellingen","Beloeil","Belsele",
		"Ben-Ahin","Bende","Berbroek","Berchem",
		"Berendrecht","Berg","Bergilers","Beringen",
		"Berlaar","Berlare","Berlingen","Berloz",
		"Berneau","Bernissart","Bersillies-l'Abbaye","Bertem",
		"Bertogne","Bertrix","Bertrée","Berzée",
		"Beselare","Betekom","Bettincourt","Beuzet",
		"Bevekom","Bevel","Bever","Bevercé",
		"Bevere","Beveren-Leie","Beveren-Roeselare","Beveren-Waas",
		"Beveren-aan-den-Ijzer","Beverlo","Beverst","Beyne-Heusay",
		"Bienne-lez-Happart","Bierbeek","Biercée","Bierges",
		"Bierghes","Bierset","Bierwart","Biesme",
		"Biesme-sous-Thuin","Biesmerée","Biez","Bihain",
		"Bikschote","Bilstain","Bilzen","Binche",
		"Binderveld","Binkom","Bioul","Bissegem",
		"Bizet","Bièvre","Blaasveld","Blaimont",
		"Blandain","Blanden","Blankenberge","Blaregnies",
		"Blaton","Blaugies","Blehen","Bleid",
		"Bleret","Blicquy","Blégny","Bléharies",
		"Bocholt","Boechout","Boekhout","Boekhoute",
		"Boezinge","Bogaarden","Bohan","Boignée",
		"Boirs","Bois-d'Haine","Bois-de-Lessines","Bois-de-Villers",
		"Bois-et-Borsu","Bolinne","Bolland","Bomal",
		"Bomal-sur-Ourthe","Bombaye","Bommershoven","Bon-Secours",
		"Boncelles","Boneffe","Bonheiden","Boninne",
		"Bonlez","Bonnert","Bonneville","Bonsin",
		"Booischot","Booitshoeke","Boom","Boorsem",
		"Boortmeerbeek","Borchtlombeek","Borgerhout","Borgloon",
		"Borlez","Borlo","Borlon","Bornem",
		"Bornival","Borsbeek","Borsbeke","Bossière",
		"Bossuit","Bossut-Gottechain","Bost","Bothey",
		"Bottelare","Bouffioulx","Bouge","Bougnies",
		"Bouillon","Bourlers","Bourseigne-Neuve","Bourseigne-Vieille",
		"Boussoit","Boussu","Boussu-en-Fagne","Boussu-lez-Walcourt",
		"Bousval","Boutersem","Bouvignes-sur-Meuse","Bouvignies",
		"Bouwel","Bovekerke","Bovelingen","Bovenistier",
		"Bovesse","Bovigny","Boëlhe","Bra",
		"Braffe","Braibant","Braine-l'Alleud","Braine-le-Château",
		"Braine-le-Comte","Braives","Brakel","Branchon",
		"Bras","Brasmenil","Brasschaat","Bray",
		"Brecht","Bredene","Bree","Breendonk",
		"Bressoux","Briegden","Brielen","Broechem",
		"Broekom","Brugelette","Brugge","Brunehaut",
		"Brussegem","Brussel","Brustem","Bruyelle",
		"Brye","Brûly","Brûly-de-Pesche","Budingen",
		"Buggenhout","Buissenal","Buissonville","Buizingen",
		"Buken","Bulskamp","Bunsbeek","Burcht",
		"Burdinne","Bure","Burg-Reuland","Burst",
		"Bury","Buvingen","Buvrinnes","Buzenol",
		"Buzet","Büllingen","Bütgenbach","Callenelle",
		"Calonne","Cambron-Casteau","Cambron-Saint-Vincent","Carlsbourg",
		"Carnières","Casteau","Castillon","Celles",
		"Cerfontaine","Chaineux","Chairière","Champion",
		"Champlon","Chanly","Chantemelle","Chapelle-lez-Herlaimont",
		"Chapelle-à-Oie","Chapelle-à-Wattines","Chapon-Seraing","Charleroi",
		"Charneux","Chassepierre","Chastre","Chastre-Villeroux-Blanmont",
		"Chastrès","Chaudfontaine","Chaumont-Gistoux",
		"Chaussée-Notre-Dame-Louvignies","Cherain","Cheratte","Chercq",
		"Chevetogne","Chevron","Chimay","Chiny","Chièvres",
		"Chokier","Châtelet","Châtelineau","Châtillon",
		"Chênée","Ciergnon","Ciney","Ciplet",
		"Ciply","Clabecq","Clavier","Clermont",
		"Clermont-sous-Huy","Cognelée","Colfontaine","Comblain-Fairon",
		"Comblain-au-Pont","Comblain-la-Tour","Conneux","Corbais",
		"Corbion","Cordes","Corenne","Cornesse",
		"Cornimont","Corroy-le-Château","Corroy-le-Grand","Corswarem",
		"Cortil-Noirmont","Cortil-Wodon","Couillet","Cour-sur-Heure",
		"Courcelles","Courrière","Court-Saint-Etienne","Couthuin",
		"Coutisse","Couture-Saint-Germain","Couvin","Cras-Avernas",
		"Crehen","Crisnée","Croix-lez-Rouveroy","Crombach",
		"Crupet","Cuesmes","Cugnon","Cul-des-Sarts",
		"Custinne","Cérexhe-Heuseux","Céroux-Mousty","Dadizele",
		"Dailly","Daknam","Dalhem","Damme",
		"Dampicourt","Dampremy","Darion","Daussois",
		"Daussoulx","Dave","Daverdisse","De Haan",
		"De Klinge","De Moeren","De Panne","De Pinte",
		"Deerlijk","Deftinge","Deinze","Denderbelle",
		"Denderhoutem","Denderleeuw","Dendermonde","Denderwindeke",
		"Dentergem","Denée","Dergneau","Dessel",
		"Desselgem","Destelbergen","Desteldonk","Deurle",
		"Deurne","Deux-Acren","Dhuy","Diepenbeek",
		"Diest","Diets-Heur","Dikkebus","Dikkele",
		"Dikkelvenne","Diksmuide","Dilbeek","Dilsen-Stokkem",
		"Dinant","Dion","Dion-Valmont","Dison",
		"Dochamps","Doel","Dohan","Doische",
		"Dolembreux","Donceel","Dongelberg","Donk",
		"Donstiennes","Dorinne","Dormaal","Dottenijs",
		"Dour","Dourbes","Dranouter","Driekapellen",
		"Drieslinter","Drogenbos","Drongen","Dréhance",
		"Dudzele","Duffel","Duisburg","Duras",
		"Durbuy","Durnal","Dworp","Eben-Emael",
		"Ebly","Ecaussinnes","Ecaussinnes-Lalaing","Ecaussinnes-d'Enghien",
		"Edegem","Edelare","Edingen","Eeklo",
		"Eernegem","Egem","Eggewaartskapelle","Eghezée",
		"Ehein","Eigenbilzen","Eindhout","Eine",
		"Eisden","Eke","Ekeren","Eksaarde",
		"Eksel","Elen","Elene","Elewijt",
		"Eliksem","Elingen","Ellemelle","Ellezelles",
		"Ellignies-Sainte-Anne","Ellignies-lez-Frasnes","Ellikom","Elouges",
		"Elsegem","Elsenborn","Elsene","Elst",
		"Elverdinge","Elversele","Emblem","Embourg",
		"Emelgem","Emines","Emptinne","Ename",
		"Engelmanshoven","Engis","Enines","Ensival",
		"Epinois","Eppegem","Eprave","Erbaut",
		"Erbisoeul","Ere","Erembodegem","Erezée",
		"Ermeton-sur-Biert","Ernage","Erneuville","Ernonheid",
		"Erondegem","Erpe","Erpe-Mere","Erpent",
		"Erpion","Erps-Kwerps","Erquelinnes","Erquennes",
		"Ertvelde","Erwetegem","Escanaffles","Esen",
		"Esneux","Esplechin","Esquelmes","Essen",
		"Essene","Estaimbourg","Estaimpuis","Estinnes",
		"Estinnes-au-Mont","Estinnes-au-Val","Etalle","Ethe",
		"Etikhove","Ettelgem","Etterbeek","Eugies",
		"Eupen","Evegnée","Evelette","Everbeek",
		"Everberg","Evere","Evergem","Evregnies",
		"Evrehailles","Eynatten","Ezemaal","Fagnolle",
		"Faimes","Falaën","Falisolle","Fallais",
		"Falmagne","Falmignoul","Familleureux","Farciennes",
		"Faulx-les-Tombes","Fauroeulx","Fauvillers","Faymonville",
		"Fays-les-Veneurs","Fayt-le-Franc","Fayt-lez-Manage","Felenne",
		"Feluy","Feneur","Fernelmont","Ferrières",
		"Feschaux","Fexhe-Slins","Fexhe-le-Haut-Clocher","Filot",
		"Finnevaux","Fize-Fontaine","Fize-le-Marsal","Flamierge",
		"Flavion","Flawinne","Fleurus","Floreffe",
		"Florennes","Florenville","Floriffoux","Florée",
		"Flostoy","Flémalle","Flémalle-Grande","Flémalle-Haute",
		"Flénu","Fléron","Flône","Focant",
		"Folx-les-Caves","Fontaine-Valmont","Fontaine-l'Evêque","Fontenelle",
		"Fontenoille","Fontenoy","Fooz","Forchies-la-Marche",
		"Forest","Forges","Forges-Philippe","Forrières",
		"Forville","Forêt","Fosse","Fosses-la-Ville",
		"Fouleng","Fourbechies","Foy-Notre-Dame","Fraipont",
		"Fraire","Fraiture","Frameries","Framont",
		"Franc-Waret","Franchimont","Francorchamps","Franière",
		"Frasnes","Frasnes-lez-Anvaing","Frasnes-lez-Buissenal",
		"Frasnes-lez-Gosselies","Freloux","Freux","Froidchapelle",
		"Froidfontaine","Froidmont","Fronville","Froyennes","Fumal",
		"Furfooz","Furnaux","Gaasbeek","Gages",
		"Gallaix","Galmaarden","Ganshoren","Gaurain-Ramecroix",
		"Gavere","Gedinne","Geel","Geer",
		"Geest-Gérompont-Petit-Rosière","Geetbets","Gelbressée","Gelinden",
		"Gellik","Gelrode","Geluveld","Geluwe",
		"Gembes","Gembloux","Gemmenich","Genappe",
		"Genk","Genly","Genoelselderen","Gent",
		"Gentbrugge","Gentinnes","Genval","Geraardsbergen",
		"Gerdingen","Gerin","Gerpinnes","Gestel",
		"Gesves","Ghislenghien","Ghlin","Ghoy",
		"Gibecq","Gierle","Gijverinkhove","Gijzegem",
		"Gijzelbrechtegem","Gijzenzele","Gilly","Gimnée",
		"Gingelom","Gistel","Gits","Givry",
		"Glabais","Glabbeek-Zuurbemde","Glain","Gleixhe",
		"Glimes","Glons","Gochenée","Godarville",
		"Godinne","Godveerdegem","Goeferdinge","Goegnies-Chaussée",
		"Goesnes","Goetsenhoven","Gomzé-Andoumont","Gondregnies",
		"Gonrieux","Gontrode","Gooik","Gors-Opleeuw",
		"Gorsem","Gosselies","Gotem","Gottem",
		"Gottignies","Gougnies","Gourdinne","Goutroux",
		"Gouvy","Gouy-lez-Piéton","Gozée","Goé",
		"Graide","Grammene","Grand-Axhe","Grand-Hallet",
		"Grand-Halleux","Grand-Leez","Grand-Manil","Grand-Rechain",
		"Grand-Reng","Grand-Rosière-Hottomont","Grandglise","Grandhan",
		"Grandmenil","Grandmetz","Grandrieu","Grandville",
		"Grandvoir","Grapfontaine","Graty","Graux",
		"Grazen","Grembergen","Grez-Doiceau","Grimbergen",
		"Grimminge","Grivegnée","Grobbendonk","Groot-Bijgaarden",
		"Groot-Gelmen","Groot-Loon","Gros-Fays","Grosage",
		"Grote-Brogel","Grote-Spouwen","Grotenberge","Gruitrode",
		"Grune","Grupont","Grâce-Berleur","Grâce-Hollogne",
		"Guignies","Guigoven","Guirsch","Gullegem",
		"Gutschoven","Gérompont","Gérouville","Haacht",
		"Haaltert","Haasdonk","Haasrode","Habay",
		"Habay-la-Neuve","Habay-la-Vieille","Habergy","Haccourt",
		"Hachy","Hacquegnies","Haillot","Haine-Saint-Paul",
		"Haine-Saint-Pierre","Hainin","Hakendover","Halanzy",
		"Halen","Hallaar","Halle","Halle-Booienhoven",
		"Halleux","Halma","Halmaal","Haltinne",
		"Ham","Ham-sur-Heure","Ham-sur-Heure-Nalinnes","Ham-sur-Sambre",
		"Hamipré","Hamme","Hamme-Mille","Hamoir",
		"Hamois","Hamont","Hamont-Achel","Hampteau",
		"Han-sur-Lesse","Handzame","Haneffe","Hannut",
		"Hannêche","Hanret","Hansbeke","Hantes-Wihéries",
		"Hanzinelle","Hanzinne","Harchies","Harelbeke",
		"Haren","Haren-Borgloon","Haren-Tongeren","Hargimont",
		"Harmignies","Harnoncourt","Harre","Harsin",
		"Harveng","Harzé","Hasselt","Hastière",
		"Hastière-Lavaux","Hastière-par-Delà","Hatrival","Haulchin",
		"Hauset","Haut-Fays","Haut-Ittre","Haut-le-Wastia",
		"Hautrage","Havay","Havelange","Haversin",
		"Havinnes","Havré","Hechtel","Hechtel-Eksel",
		"Heer","Heers","Hees","Heestert",
		"Heffen","Heikruis","Heindonk","Heinsch",
		"Heist-aan-Zee","Heist-op-den-Berg","Hekelgem","Heks",
		"Helchteren","Heldergem","Helen-Bos","Helkijn",
		"Hellebecq","Hemelveerdegem","Hemiksem","Hemptinne",
		"Hemptinne-lez-Florennes","Hendrieken","Henis","Hennuyères",
		"Henri-Chapelle","Henripont","Hensies","Heppen",
		"Heppenbach","Heppignies","Herbeumont","Herchies",
		"Herderen","Herdersem","Herent","Herentals",
		"Herenthout","Herfelingen","Hergenrath","Herk-de-Stad",
		"Hermalle-sous-Argenteau","Hermalle-sous-Huy","Hermeton-sur-Meuse",
		"Hermée","Herne","Herquegies","Herseaux","Herselt",
		"Herstal","Herstappe","Hertain","Herten",
		"Hertsberge","Herve","Herzele","Heule",
		"Heure","Heure-le-Romain","Heurne","Heusden",
		"Heusden-Zolder","Heusy","Heuvelland","Hever",
		"Heverlee","Heyd","Hillegem","Hingene",
		"Hingeon","Hives","Hoboken","Hodeige",
		"Hodister","Hody","Hoegaarden","Hoeilaart",
		"Hoeke","Hoelbeek","Hoeleden","Hoepertingen",
		"Hoeselt","Hoevenen","Hofstade","Hogne",
		"Hognoul","Hollain","Hollange","Hollebeke",
		"Hollogne-aux-Pierres","Hollogne-sur-Geer","Holsbeek","Hombeek",
		"Hombourg","Hompré","Hondelange","Honnay",
		"Honnelles","Hooglede","Hoogstade","Hoogstraten",
		"Horebeke","Horion-Hozémont","Hornu","Horpmaal",
		"Horrues","Hotton","Houdemont","Houdeng-Aimeries",
		"Houdeng-Goegnies","Houdremont","Houffalize","Hour",
		"Housse","Houtain-Saint-Siméon","Houtain-le-Val","Houtaing",
		"Houtave","Houtem","Houthalen","Houthalen-Helchteren",
		"Houthem","Houthulst","Houtvenne","Houwaart",
		"Houx","Houyet","Hove","Hoves",
		"Howardries","Huccorgne","Huise","Huissignies",
		"Huizingen","Huldenberg","Hulshout","Hulsonniaux",
		"Hulste","Humain","Humbeek","Hundelgem",
		"Huppaye","Huy","Hyon","Hélécine",
		"Hérinnes-lez-Pecq","Héron","Hévillers","Ichtegem",
		"Iddergem","Idegem","Ieper","Impe",
		"Incourt","Ingelmunster","Ingooigem","Irchonwelz",
		"Isières","Isnes","Itegem","Itterbeek",
		"Ittre","Ivoz-Ramet","Izegem","Izel",
		"Izenberge","Izier","Jabbeke","Jalhay",
		"Jallet","Jamagne","Jambes","Jamiolle",
		"Jamioulx","Jamoigne","Jandrain-Jandrenouille","Jauche",
		"Jauchelette","Javingue","Jehay","Jehonville",
		"Jemappes","Jemelle","Jemeppe-sur-Meuse","Jemeppe-sur-Sambre",
		"Jeneffe","Jesseren","Jette","Jeuk",
		"Jodoigne","Jodoigne-Souveraine","Jollain-Merlin","Joncret",
		"Julémont","Jumet","Jupille-sur-Meuse","Juprelle",
		"Jurbise","Juseret","Kaaskerke","Kachtem",
		"Kaggevinne","Kain","Kalken","Kallo",
		"Kallo-Kieldrecht","Kalmthout","Kampenhout","Kanegem",
		"Kanne","Kapelle-op-den-Bos","Kapellen","Kaprijke",
		"Kaster","Kasterlee","Kaulille","Keerbergen",
		"Keiem","Kelmis","Kemexhe","Kemmel",
		"Kemzeke","Kerkhove","Kerkom","Kerkom-bij-Sint-Truiden",
		"Kerksken","Kermt","Kerniel","Kersbeek-Miskom",
		"Kessel","Kessel-Lo","Kessenich","Kester",
		"Kettenis","Keumiée","Kieldrecht","Kinrooi",
		"Klein-Gelmen","Kleine-Brogel","Kleine-Spouwen","Klemskerke",
		"Klerken","Kluisbergen","Kluizen","Knesselare",
		"Knokke","Knokke-Heist","Kobbegem","Koekelare",
		"Koekelberg","Koersel","Koksijde","Kolmont-Borgloon",
		"Kolmont-Tongeren","Komen","Komen-Waasten","Koningshooikt",
		"Koninksem","Kontich","Kooigem","Koolkerke",
		"Koolskamp","Korbeek-Dijle","Korbeek-Lo","Kortemark",
		"Kortenaken","Kortenberg","Kortessem","Kortijs",
		"Kortrijk","Kortrijk-Dutsel","Kozen","Kraainem",
		"Krombeke","Kruibeke","Kruishoutem","Kumtich",
		"Kuringen","Kuttekoven","Kuurne","Kwaadmechelen",
		"Kwaremont","La","La Bruyère","La Glanerie",
		"La Gleize","La Hestre","La Hulpe","La Louvière",
		"La bouverie","La-Roche-en-Ardenne","Laakdal","Laar",
		"Laarne","Labuissière","Lacuisine","Ladeuze",
		"Laforêt","Lahamaide","Laken","Lamain",
		"Lambermont","Lambusart","Lamine","Lamontzée",
		"Lamorteau","Lampernisse","Lanaken","Lanaye",
		"Landegem","Landelies","Landen","Landenne",
		"Landskouter","Laneffe","Langdorp","Langemark",
		"Langemark-Poelkapelle","Lanklaar","Lanquesaint","Lantin",
		"Lantremange","Laplaigne","Lapscheure","Lasne",
		"Lasne-Chapelle-Saint-Lambert","Lathuy","Latinne","Latour",
		"Lauw","Lauwe","Lavacherie","Lavaux-Sainte-Anne",
		"Lavoir","Le Mesniel","Le Roeulx","Le Roux",
		"Lebbeke","Lede","Ledeberg","Ledegem",
		"Leefdaal","Leerbeek","Leernes","Leers-Nord",
		"Leers-et-Fosteau","Leest","Leeuwergem","Leffinge",
		"Leignon","Leisele","Leke","Lembeek",
		"Lembeke","Lemberge","Lendelede","Lennik",
		"Lens","Lens-Saint-Remy","Lens-Saint-Servais","Lens-sur-Geer",
		"Leopoldsburg","Les Avins","Les Bons","Les Bulles",
		"Les Hayons","Les Waleffes","Lesdain","Lessines",
		"Lessive","Lesterny","Lesve","Lettelingen",
		"Letterhoutem","Leugnies","Leupegem","Leut",
		"Leuven","Leuze","Leuze-en-Hainaut","Leval-Chaudeville",
		"Leval-Trahegnies","Liberchies","Libin","Libramont",
		"Libramont-Chevigny","Lichtaart","Lichtervelde","Liedekerke",
		"Lieferinge","Lier","Lierde","Lierneux",
		"Liernu","Liers","Liezele","Ligne",
		"Ligney","Ligny","Lille","Lillo",
		"Lillois-Witterzée","Limal","Limbourg","Limelette",
		"Limerlé","Limont","Lincent","Linden",
		"Linkebeek","Linkhout","Linsmeau","Lint",
		"Linter","Lippelo","Lisogne","Lissewege",
		"Lives-sur-Meuse","Lixhe","Liège","Lo",
		"Lo-Reninge","Lobbes","Lochristi","Lodelinsart",
		"Loenhout","Loker","Lokeren","Loksbergen",
		"Lombardsijde","Lombise","Lommel","Lommersweiler",
		"Lompret","Lomprez","Loncin","Londerzeel",
		"Longchamps","Longlier","Longueville","Longvilly",
		"Lontzen","Lonzée","Loonbeek","Loppem",
		"Lorcé","Lot","Lotenhulle","Louette-Saint-Denis",
		"Louette-Saint-Pierre","Loupoigne","Louvain-la-Neuve",
		"Louveigné","Lovendegem","Lovenjoel","Loverval","Loyers",
		"Lubbeek","Luingne","Lummen","Lustin",
		"Luttre","Léglise","Maarke-Kerkem","Maarkedal",
		"Maaseik","Maasmechelen","Mabompré","Machelen",
		"Macon","Macquenoise","Maffe","Maffle",
		"Magnée","Maillen","Mainvault","Maisières",
		"Maissin","Maizeret","Mal","Maldegem",
		"Malderen","Malempré","Malle","Malmedy",
		"Malonne","Malvoisin","Malèves-Sainte-Marie-Wastines","Manage",
		"Manderfeld","Manhay","Mannekensvere","Maransart",
		"Marbais","Marbaix","Marbehan","Marche-en-Famenne",
		"Marche-les-Dames","Marche-lez-Ecaussinnes","Marchienne-au-Pont",
		"Marchin","Marchipont","Marchovelette","Marcinelle","Marcourt",
		"Marenne","Mariakerke","Mariekerke","Mariembourg",
		"Marilles","Mark","Marke","Markegem",
		"Marneffe","Marquain","Martelange","Martenslinde",
		"Martouzin-Neuville","Masbourg","Masnuy-Saint-Jean",
		"Masnuy-Saint-Pierre","Massemen","Massenhoven","Matagne-la-Grande",
		"Matagne-la-Petite","Mater","Maubray","Maulde","Maurage",
		"Mazenzele","Mazy","Mazée","Mechelen",
		"Mechelen-Bovelingen","Mechelen-aan-de-Maas","Meeffe",
		"Meensel-Kiezegem","Meer","Meerbeek","Meerbeke","Meerdonk",
		"Meerhout","Meerle","Meeswijk","Meetkerke",
		"Meeuwen","Meeuwen-Gruitrode","Mehaigne","Meigem",
		"Meilegem","Meise","Meix-devant-Virton","Meix-le-Tige",
		"Melden","Meldert","Melen","Melkwezer",
		"Melle","Mellery","Melles","Mellet",
		"Mellier","Melsbroek","Melsele","Melsen",
		"Membach","Membre","Membruggen","Mendonk",
		"Menen","Merbes-Sainte-Marie","Merbes-le-Château","Merchtem",
		"Merdorp","Mere","Merelbeke","Merendree",
		"Merkem","Merksem","Merksplas","Merlemont",
		"Mesen","Meslin-l'Evêque","Mesnil-Eglise","Mesnil-Saint-Blaise",
		"Mespelare","Messancy","Messelbroek","Mesvin",
		"Mettekoven","Mettet","Meulebeke","Meux",
		"Meyerode","Michelbeke","Micheroux","Middelburg",
		"Middelkerke","Mielen-boven-Aalst","Mignault","Millen",
		"Milmort","Minderhout","Mirwart","Miécret",
		"Modave","Moelingen","Moen","Moerbeke",
		"Moerbeke-Waas","Moere","Moerkerke","Moerzeke",
		"Moeskroen","Moha","Mohiville","Moignelée",
		"Moircy","Mol","Molenbaix","Molenbeek-Wersbeek",
		"Molenbeersel","Molenstede","Mollem","Momalle",
		"Momignies","Monceau-Imbrechies","Monceau-en-Ardenne",
		"Monceau-sur-Sambre","Mons","Mons-lez-Liège","Monstreux","Mont",
		"Mont-Gauthier","Mont-Saint-André","Mont-Saint-Aubert",
		"Mont-Saint-Guibert","Mont-Sainte-Aldegonde","Mont-Sainte-Geneviève",
		"Mont-de-l'Enclus","Mont-sur-Marchienne","Montbliart","Montegnée",
		"Montenaken","Montignies-Saint-Christophe","Montignies-lez-Lens",
		"Montignies-sur-Roc","Montignies-sur-Sambre","Montigny-le-Tilleul",
		"Montleban","Montroeul-au-Bois","Montroeul-sur-Haine","Montzen",
		"Moorsel","Moorsele","Moorslede","Moortsele",
		"Mopertingen","Moregem","Moresnet","Morhet",
		"Morialmé","Morkhoven","Morlanwelz","Morlanwelz-Mariemont",
		"Mormont","Mornimont","Mortier","Mortroux",
		"Mortsel","Morville","Moulbaix","Mourcourt",
		"Moustier","Moustier-sur-Sambre","Mouzaive","Moxhe",
		"Mozet","Muizen","Mullem","Munkzwalm",
		"Muno","Munsterbilzen","Munte","Musson",
		"Mussy-la-Ville","My","Méan","Mélin",
		"Mévergnies-lez-Lens","Naast","Nadrin","Nafraiture",
		"Nalinnes","Namur","Namêche","Nandrin",
		"Naninne","Naomé","Nassogne","Natoye",
		"Nazareth","Neder-over-Heembeek","Nederboelare","Nederbrakel",
		"Nederename","Nederhasselt","Nederokkerzeel","Nederzwalm-Hermelgem",
		"Neerglabbeek","Neerharen","Neerhespen","Neerheylissem",
		"Neerijse","Neerlanden","Neerlinter","Neeroeteren",
		"Neerpelt","Neerrepen","Neervelp","Neerwaasten",
		"Neerwinden","Neigem","Nerem","Nessonvaux",
		"Nethen","Nettinne","Neu-Moresnet","Neufchâteau",
		"Neufmaison","Neufvilles","Neupré","Neuville",
		"Neuville-en-Condroz","Nevele","Niel","Niel-bij-As",
		"Niel-bij-Sint-Truiden","Nieuwenhove","Nieuwenrode","Nieuwerkerken",
		"Nieuwkapelle","Nieuwkerke","Nieuwkerken-Waas","Nieuwmunster",
		"Nieuwpoort","Nieuwrode","Nijlen","Nil-Saint-Vincent-Saint-Martin",
		"Nimy","Ninove","Nismes","Nivelles",
		"Niverlée","Nives","Nobressart","Nodebais",
		"Noduwez","Noirchain","Noirefontaine","Noiseux",
		"Nokere","Nollevaux","Noorderwijk","Noordschote",
		"Nossegem","Nothomb","Nouvelles","Noville",
		"Noville-les-Bois","Noville-sur-Méhaigne","Nukerke","Néchin",
		"Obaix","Obigies","Obourg","Ochamps",
		"Ocquier","Odeigne","Odeur","Oedelem",
		"Oekene","Oelegem","Oeren","Oeselgem",
		"Oetingen","Oeudeghien","Oevel","Offagne",
		"Ogy","Ohain","Ohey","Oignies-en-Thiérache",
		"Oisquercq","Oizy","Okegem","Olen",
		"Oleye","Ollignies","Olloy-sur-Viroin","Olmen",
		"Olne","Olsene","Omal","Ombret",
		"Omezée","On","Onhaye","Onkerzele",
		"Onnezies","Onoz","Onze-Lieve-Vrouw-Lombeek",
		"Onze-Lieve-Vrouw-Waver","Ooigem","Ooike","Oombergen","Oorbeek",
		"Oordegem","Oostakker","Oostduinkerke","Oosteeklo",
		"Oostende","Oosterzele","Oostham","Oostkamp",
		"Oostkerke-Damme","Oostkerke-Diksmuide","Oostmalle","Oostnieuwkerke",
		"Oostrozebeke","Oostvleteren","Oostwinkel","Opbrakel",
		"Opdorp","Opglabbeek","Opgrimbie","Ophain-Bois-Seigneur-Isaac",
		"Ophasselt","Opheers","Opheylissem","Ophoven",
		"Opitter","Oplinter","Opoeteren","Opont",
		"Opprebais","Oppuurs","Opvelp","Opwijk",
		"Orbais","Orchimont","Orcq","Ordingen",
		"Oret","Oreye","Orgeo","Ormeignies",
		"Orp-Jauche","Orp-le-Grand","Orroir","Orsmaal-Gussenhoven",
		"Ortho","Ostiches","Otegem","Oteppe",
		"Othée","Otrange","Ottenburg","Ottergem",
		"Ottignies","Ottignies-Louvain-la-Neuve","Oud-Heverlee",
		"Oud-Turnhout","Oudegem","Oudekapelle","Oudenaarde","Oudenaken",
		"Oudenburg","Oudergem","Ouffet","Ougrée",
		"Oupeye","Outer","Outgaarden","Outrelouxhe",
		"Outrijve","Ouwegem","Overboelare","Overhespen",
		"Overijse","Overmere","Overpelt","Overrepen",
		"Overwinden","Paal","Paifve","Pailhe",
		"Paliseul","Pamel","Papignies","Parike",
		"Passendale","Patignies","Paturages","Paulatem",
		"Pecq","Peer","Peissant","Pellaines",
		"Pellenberg","Pepingen","Pepinster","Perk",
		"Pervijze","Perwez","Perwez-Haillot","Pesche",
		"Pessoux","Petegem-aan-de-Leie","Petegem-aan-de-Schelde","Petigny",
		"Petit-Fays","Petit-Hallet","Petit-Rechain","Petit-Roeulx-lez-Braine",
		"Petit-Roeulx-lez-Nivelles","Petit-Thier","Petite-Chapelle","Peutie",
		"Philippeville","Pipaix","Piringen","Pironchamps",
		"Pittem","Piéton","Piétrain","Piétrebais",
		"Plainevaux","Plancenoit","Ploegsteert","Plombières",
		"Poederlee","Poeke","Poelkapelle","Poesele",
		"Pollare","Polleur","Pollinkhove","Pommeroeul",
		"Pondrôme","Pont-de-Loup","Pont-à-Celles","Pontillas",
		"Poperinge","Poppel","Popuelles","Porcheresse",
		"Pottes","Poucet","Poulseur","Poupehan",
		"Pousset","Presgaux","Presles","Profondeville",
		"Proven","Pry","Pulderbos","Pulle",
		"Purnode","Pussemange","Putte","Puurs",
		"Péronnes-lez-Antoing","Péronnes-lez-Binche","Péruwelz","Quaregnon",
		"Quartes","Quenast","Queue-du-Bois","Quevaucamps",
		"Quiévrain","Quévy","Quévy-le-Grand","Quévy-le-Petit",
		"Rachecourt","Racour","Raeren","Ragnies",
		"Rahier","Ramegnies","Ramegnies-Chin","Ramelot",
		"Ramillies-Offus","Ramsdonk","Ramsel","Ramskapelle-Knokke-Heist",
		"Ramskapelle-Nieuwpoort","Rance","Ransart","Ransberg",
		"Ranst","Ravels","Rebaix","Rebecq",
		"Rebecq-Rognon","Recht","Recogne","Redu",
		"Reet","Rekem","Rekkem","Relegem",
		"Remagne","Remersdaal","Remicourt","Rendeux",
		"Reninge","Reningelst","Renlies","Reppel",
		"Ressaix","Ressegem","Resteigne","Retie",
		"Retinne","Reuland","Rhisnes","Richelle",
		"Riemst","Rienne","Rijkel","Rijkevorsel",
		"Rijkhoven","Rijmenam","Riksingen","Rillaar",
		"Rivière","Rixensart","Rièzes","Robechies",
		"Robelmont","Robertville","Roborst","Rochefort",
		"Rochehaut","Rocherath","Roclenge-sur-Geer","Rocourt",
		"Roesbrugge-Haringe","Roeselare","Rognée","Roisin",
		"Roksem","Rollegem","Rollegem-Kapelle","Roloux",
		"Roly","Romedenne","Romershoven","Romerée",
		"Romsée","Rongy","Ronquières","Ronse",
		"Ronsele","Roosbeek","Roosdaal","Roselies",
		"Rosières","Rosmeer","Rosoux-Crenwick","Rossignol",
		"Rosée","Rotem","Rotheux-Rimière","Rotselaar",
		"Roucourt","Rouveroy","Rouvreux","Rouvroy",
		"Roux","Roux-Miroir","Roy","Rozebeke",
		"Ruddervoorde","Ruette","Ruien","Ruisbroek",
		"Ruiselede","Rukkelingen-Loon","Rulles","Rumbeke",
		"Rumes","Rumillies","Rummen","Rumsdorp",
		"Rumst","Runkelen","Rupelmonde","Russeignies",
		"Rutten","Rèves","Saint-Amand","Saint-André",
		"Saint-Aubin","Saint-Denis","Saint-Denis-Bovesse",
		"Saint-Georges-sur-Meuse","Saint-Germain","Saint-Ghislain",
		"Saint-Gérard","Saint-Géry","Saint-Hubert","Saint-Jean-Geest",
		"Saint-Léger","Saint-Marc","Saint-Mard","Saint-Martin","Saint-Maur",
		"Saint-Médard","Saint-Nicolas","Saint-Pierre","Saint-Remy",
		"Saint-Remy-Geest","Saint-Sauveur","Saint-Servais","Saint-Symphorien",
		"Saint-Séverin","Saint-Vaast","Saint-Vincent","Sainte-Cécile",
		"Sainte-Marie-Chevigny","Sainte-Marie-sur-Semois","Sainte-Ode",
		"Saintes","Saive","Salles","Samart","Sambreville","Samrée",
		"Sankt-Vith","Sars-la-Bruyère","Sars-la-Buissière","Sart-Bernard",
		"Sart-Custinne","Sart-Dames-Avelines","Sart-Eustache",
		"Sart-Saint-Laurent","Sart-en-Fagne","Sart-lez-Spa","Sautin",
		"Sautour","Sauvenière","Schaarbeek","Schaffen","Schalkhoven",
		"Schaltin","Schelderode","Scheldewindeke","Schelle",
		"Schellebelle","Schendelbeke","Schepdaal","Scherpenheuvel",
		"Scherpenheuvel-Zichem","Schilde","Schoonaarde","Schore",
		"Schorisse","Schoten","Schriek","Schuiferskapelle",
		"Schulen","Schönberg","Sclayn","Scy",
		"Seilles","Seloignes","Semmerzake","Seneffe",
		"Sensenruth","Seny","Senzeille","Septon",
		"Seraing","Seraing-le-Château","Serinchamps","Serskamp",
		"Serville","Sibret","Signeulx","Sijsele",
		"Silenrieux","Silly","Sinaai-Waas","Sinsin",
		"Sint-Agatha-Berchem","Sint-Agatha-Rode","Sint-Amands",
		"Sint-Amandsberg","Sint-Andries","Sint-Antelinks","Sint-Baafs-Vijve",
		"Sint-Blasius-Boekel","Sint-Denijs","Sint-Denijs-Boekel",
		"Sint-Denijs-Westrem","Sint-Eloois-Vijve","Sint-Eloois-Winkel",
		"Sint-Genesius-Rode","Sint-Gillis","Sint-Gillis-Waas",
		"Sint-Gillis-bij-Dendermonde","Sint-Goriks-Oudenhove",
		"Sint-Huibrechts-Hern","Sint-Huibrechts-Lille","Sint-Jacobs-Kapelle",
		"Sint-Jan","Sint-Jan-in-Eremo","Sint-Jans-Molenbeek",
		"Sint-Job-in-'t-Goor","Sint-Joost-ten-Node","Sint-Joris-Beernem",
		"Sint-Joris-Nieuwpoort","Sint-Joris-Weert","Sint-Joris-Winge",
		"Sint-Katelijne-Waver","Sint-Katherina-Lombeek",
		"Sint-Kornelis-Horebeke","Sint-Kruis","Sint-Kruis-Winkel",
		"Sint-Kwintens-Lennik","Sint-Lambrechts-Herk",
		"Sint-Lambrechts-Woluwe","Sint-Laureins","Sint-Laureins-Berchem",
		"Sint-Lenaarts","Sint-Lievens-Esse","Sint-Lievens-Houtem",
		"Sint-Margriete","Sint-Margriete-Houtem","Sint-Maria-Horebeke",
		"Sint-Maria-Latem","Sint-Maria-Lierde","Sint-Maria-Oudenhove-Brakel",
		"Sint-Maria-Oudenhove-Zottegem","Sint-Martens-Bodegem",
		"Sint-Martens-Latem","Sint-Martens-Leerne","Sint-Martens-Lennik",
		"Sint-Martens-Lierde","Sint-Martens-Voeren","Sint-Michiels",
		"Sint-Niklaas","Sint-Pauwels","Sint-Pieters-Kapelle",
		"Sint-Pieters-Leeuw","Sint-Pieters-Rode","Sint-Pieters-Voeren",
		"Sint-Pieters-Woluwe","Sint-Rijkers","Sint-Stevens-Woluwe",
		"Sint-Truiden","Sint-Ulriks-Kapelle","Sippenaeken","Sirault","Sivry",
		"Sivry-Rance","Sleidinge","Slijpe","Slins","Sluizen",
		"Smeerebbe-Vloerzegem","Smetlede","Smuid","Snaaskerke",
		"Snellegem","Soheit-Tinlot","Sohier","Soignies",
		"Soiron","Solre-Saint-Géry","Solre-sur-Sambre","Sombreffe",
		"Somme-Leuze","Sommethonne","Sommière","Somzée",
		"Sorinne-la-Longue","Sorinnes","Sorée","Sosoye",
		"Sougné-Remouchamps","Soulme","Soumagne","Soumoy",
		"Sourbrodt","Souvret","Sovet","Soy",
		"Soye","Spa","Spalbeek","Spermalie",
		"Spiennes","Spiere","Spiere-Helkijn","Spontin",
		"Spouwen","Sprimont","Spy","Stabroek",
		"Staden","Stalhille","Stambruges","Stave",
		"Stavele","Stavelot","Steendorp","Steenhuffel",
		"Steenhuize-Wijnhuize","Steenkerke","Steenkerque","Steenokkerzeel",
		"Stekene","Stembert","Stene","Sterrebeek",
		"Stevoort","Stokrooie","Stoumont","Straimont",
		"Strijpen","Strijtem","Strombeek-Bever","Strée",
		"Strée-lez-Huy","Strépy-Bracquegnies","Stuivekenskerke","Suarlée",
		"Sugny","Surice","Suxy","Sélange",
		"Tailles","Taintignies","Tamines","Tarcienne",
		"Tavier","Taviers","Tavigny","Tellin",
		"Templeuve","Temploux","Temse","Tenneville",
		"Teralfene","Terhagen","Termes","Ternat",
		"Tertre","Tervuren","Terwagne","Tessenderlo",
		"Testelt","Teuven","Theux","Thiaumont",
		"Thieu","Thieulain","Thieusies","Thimister",
		"Thimister-Clermont","Thimougies","Thiméon","Thines",
		"Thirimont","Thisnes","Thommen","Thon",
		"Thorembais-Saint-Trond","Thorembais-les-Béguines",
		"Thoricourt","Thuillies","Thuin","Thulin","Thumaide","Thy-le-Bauduin",
		"Thy-le-Château","Thynes","Thys","Tiegem",
		"Tielen","Tielrode","Tielt","Tielt-Winge",
		"Tienen","Tignée","Tihange","Tildonk",
		"Tilff","Tillet","Tilleur","Tillier",
		"Tilly","Tinlot","Tintange","Tintigny",
		"Tisselt","Toernich","Tohogne","Tollembeek",
		"Tongeren","Tongerlo","Tongre-Notre-Dame","Tongre-Saint-Martin",
		"Tongrinne","Tontelange","Torgny","Torhout",
		"Tourinne","Tourinnes-Saint-Lambert","Tournai","Tournay",
		"Tourpes","Transinne","Trazegnies","Treignes",
		"Trembleur","Tremelo","Trivières","Trognée",
		"Trois-Ponts","Trooz","Tubize","Turnhout",
		"Ucimont","Uikhoven","Uitbergen","Uitkerke",
		"Ukkel","Ulbeek","Upigny","Ursel",
		"Vaalbeek","Val-Meer","Vance","Varendonk",
		"Varsenare","Vaucelles","Vaulx","Vaulx-lez-Chimay",
		"Vaux-Chavanne","Vaux-et-Borset","Vaux-lez-Rosières",
		"Vaux-sous-Chèvremont","Vaux-sur-Sûre","Vechmaal","Vedrin","Veerle",
		"Velaine-sur-Sambre","Velaines","Veldegem","Veldwezelt",
		"Vellereille-le-Sec","Vellereille-les-Brayeux","Velm","Velroux",
		"Veltem-Beisem","Velzeke-Ruddershove","Vencimont","Vergnies",
		"Verlaine","Verlée","Verrebroek","Vertrijk",
		"Verviers","Vesqueville","Veulen","Veurne",
		"Vezin","Vezon","Viane","Vichte",
		"Vielsalm","Viemme","Viersel","Vierset-Barse",
		"Vierves-sur-Viroin","Viesville","Vieux-Genappe","Vieux-Waleffe",
		"Vieuxville","Villance","Ville-Pommeroeul","Ville-en-Hesbaye",
		"Ville-sur-Haine","Villerot","Villers-Deux-Eglises",
		"Villers-Notre-Dame","Villers-Perwin","Villers-Poterie",
		"Villers-Saint-Amand","Villers-Saint-Ghislain",
		"Villers-Saint-Siméon","Villers-Sainte-Gertrude","Villers-aux-Tours",
		"Villers-devant-Orval","Villers-en-Fagne","Villers-l'Evêque",
		"Villers-la-Bonne-Eau","Villers-la-Loue","Villers-la-Tour",
		"Villers-la-Ville","Villers-le-Bouillet","Villers-le-Gambon",
		"Villers-le-Peuplier","Villers-le-Temple","Villers-lez-Heest",
		"Villers-sur-Lesse","Villers-sur-Semois","Vilvoorde","Vinalmont",
		"Vinderhoute","Vinkem","Vinkt","Virelles","Virginal-Samme",
		"Viroinval","Virton","Vissenaken","Visé",
		"Vitrival","Vivegnis","Vivy","Vladslo",
		"Vlamertinge","Vlekkem","Vleteren","Vlezenbeek",
		"Vliermaal","Vliermaalroot","Vlierzele","Vlijtingen",
		"Vlimmeren","Vlissegem","Vloesberg","Vodecée",
		"Vodelée","Voeren","Vogenée","Volkegem",
		"Vollezele","Vonêche","Voorde","Voormezele",
		"Voort","Voroux-Goreux","Voroux-lez-Liers","Vorselaar",
		"Vorsen","Vorst","Vosselaar","Vosselare",
		"Vossem","Vottem","Vrasene","Vremde",
		"Vreren","Vresse-sur-Semois","Vroenhoven","Vucht",
		"Vurste","Vyle-et-Tharoul","Waanrode","Waarbeke",
		"Waardamme","Waarloos","Waarmaarde","Waarschoot",
		"Waasmont","Waasmunster","Waasten","Wachtebeke",
		"Wadelincourt","Wagnelée","Waha","Waillet",
		"Wakken","Walcourt","Walem","Walhain",
		"Walhain-Saint-Paul","Walhorn","Walsbets","Walshoutem",
		"Waltwilder","Wambeek","Wancennes","Wandre",
		"Wanfercée-Baulet","Wange","Wangenies","Wanlin",
		"Wanne","Wannebecq","Wannegem-Lede","Wansin",
		"Wanze","Wanzele","Warchin","Warcoing",
		"Wardin","Waregem","Waremme","Waret-l'Evêque",
		"Waret-la-Chaussée","Warisoulx","Warnant","Warnant-Dreye",
		"Warquignies","Warsage","Warzée","Wasmes",
		"Wasmes-Audemez-Briffoeil","Wasmuel","Wasseiges","Waterland-Oudeman",
		"Waterloo","Watermaal-Bosvoorde","Watervliet","Watou",
		"Wattripont","Waudrez","Waulsort","Wauthier-Braine",
		"Waver","Wavreille","Wayaux","Ways",
		"Webbekom","Wechelderzande","Weelde","Weerde",
		"Weert","Wegnez","Weillen","Weismes",
		"Welden","Welkenraedt","Welle","Wellen",
		"Wellin","Wemmel","Wenduine","Werbomont",
		"Werchter","Werken","Werm","Wervik",
		"Wespelaar","Westende","Westerlo","Westkapelle",
		"Westkerke","Westmalle","Westmeerbeek","Westouter",
		"Westrem","Westrozebeke","Westvleteren","Wetteren",
		"Wevelgem","Wez-Velvain","Wezemaal","Wezembeek-Oppem",
		"Wezeren","Wibrin","Wichelen","Widooie",
		"Wiekevorst","Wielsbeke","Wierde","Wiers",
		"Wiesme","Wieze","Wihogne","Wihéries",
		"Wijchmaal","Wijer","Wijgmaal","Wijnegem",
		"Wijshagen","Wijtschate","Wilderen","Willaupuis",
		"Willebringen","Willebroek","Willemeau","Willerzie",
		"Wilrijk","Wilsele","Wilskerke","Wimmertingen",
		"Winenne","Wingene","Winksele","Wintershoven",
		"Witry","Wodecq","Woesten","Wolkrange",
		"Wolvertem","Wommelgem","Wommersom","Wonck",
		"Wondelgem","Wontergem","Wortegem","Wortegem-Petegem",
		"Wortel","Woubrechtegem","Woumen","Wulpen",
		"Wulvergem","Wulveringem","Wuustwezel","Wépion",
		"Wéris","Xhendelesse","Xhendremael","Xhoris",
		"Yernée-Fraineux","Yves-Gomezée","Yvoir","Zaffelare",
		"Zandbergen","Zande","Zandhoven","Zandvliet",
		"Zandvoorde-Oostende","Zandvoorde-Zonnebeke","Zarlardinge","Zarren",
		"Zaventem","Zedelgem","Zeebrugge","Zegelsem",
		"Zele","Zelem","Zellik","Zelzate",
		"Zemst","Zepperen","Zerkegem","Zevekote",
		"Zeveneken","Zeveren","Zevergem","Zichem",
		"Zichen-Zussen-Bolder","Zillebeke","Zingem","Zoerle-Parwijs",
		"Zoersel","Zolder","Zomergem","Zonhoven",
		"Zonnebeke","Zonnegem","Zottegem","Zoutenaaie",
		"Zoutleeuw","Zuidschote","Zuienkerke","Zulte",
		"Zulzeke","Zutendaal","Zwalm","Zwevegem",
		"Zwevezele","Zwijnaarde","Zwijndrecht","Zétrud-Lumay",
		"l'Escaillère"
    )

    provinces = (
		"Antwerpen", "Henegouwen", "Limburg", "Luik", "Luxemburg", "Namen",
		"Oost-Vlaanderen", "Vlaams-Brabant", "Waals-Brabant", "West-Vlaanderen"
    )

    street_name_formats = (
        '{{first_name}}{{street_suffix}}',
    )

    street_address_formats = (
        '{{street_name}} {{building_number}}',
    )

    address_formats = (
        "{{street_address}}\n{{postcode}}\n{{city}}","{{street_address}}\n{{postcode}} {{city}}",
    )

    def province(self):
        return self.random_element(self.provinces)

    def city(self):
        return self.random_element(self.cities)



from __future__ import unicode_literals

from ..en import Provider as AddressProvider


class Provider(AddressProvider):

    city_prefixes = ('North', 'East', 'West', 'South', 'New', 'Lake', 'Port',
                     'St.')

    city_suffixes = ('town', 'ton', 'land', 'ville', 'berg', 'burgh',
                     'borough', 'bury', 'view', 'port', 'mouth', 'stad',
                     'furt', 'chester', 'mouth', 'fort', 'haven', 'side',
                     'shire')

    building_number_formats = ('###', '##', '#')

    street_suffixes = (
        'Access', 'Alley', 'Alleyway', 'Amble', 'Anchorage', 'Approach',
        'Arcade', 'Artery', 'Avenue', 'Basin', 'Beach', 'Bend', 'Block',
        'Boulevard', 'Brace', 'Brae', 'Break', 'Bridge', 'Broadway', 'Brow',
        'Bypass', 'Byway', 'Causeway', 'Centre', 'Centreway', 'Chase',
        'Circle', 'Circlet', 'Circuit', 'Circus', 'Close', 'Colonnade',
        'Common', 'Concourse', 'Copse', 'Corner', 'Corso', 'Court',
        'Courtyard', 'Cove', 'Crescent', 'Crest', 'Cross', 'Crossing',
        'Crossroad', 'Crossway', 'Cruiseway', 'Cul-de-sac', 'Cutting', 'Dale',
        'Dell', 'Deviation', 'Dip', 'Distributor', 'Drive', 'Driveway', 'Edge',
        'Elbow', 'End', 'Entrance', 'Esplanade', 'Estate', 'Expressway',
        'Extension', 'Fairway', 'Fire Track', 'Firetrail', 'Flat', 'Follow',
        'Footway', 'Foreshore', 'Formation', 'Freeway', 'Front', 'Frontage',
        'Gap', 'Garden', 'Gardens', 'Gate', 'Gates', 'Glade', 'Glen', 'Grange',
        'Green', 'Ground', 'Grove', 'Gully', 'Heights', 'Highroad', 'Highway',
        'Hill', 'Interchange', 'Intersection', 'Junction', 'Key', 'Landing',
        'Lane', 'Laneway', 'Lees', 'Line', 'Link', 'Little', 'Lookout', 'Loop',
        'Lower', 'Mall', 'Meander', 'Mew', 'Mews', 'Motorway', 'Mount', 'Nook',
        'Outlook', 'Parade', 'Park', 'Parklands', 'Parkway', 'Part', 'Pass',
        'Path', 'Pathway', 'Piazza', 'Place', 'Plateau', 'Plaza', 'Pocket',
        'Point', 'Port', 'Promenade', 'Quad', 'Quadrangle', 'Quadrant', 'Quay',
        'Quays', 'Ramble', 'Ramp', 'Range', 'Reach', 'Reserve', 'Rest',
        'Retreat', 'Ride', 'Ridge', 'Ridgeway', 'Right Of Way', 'Ring', 'Rise',
        'River', 'Riverway', 'Riviera', 'Road', 'Roads', 'Roadside', 'Roadway',
        'Ronde', 'Rosebowl', 'Rotary', 'Round', 'Route', 'Row', 'Rue', 'Run',
        'Service Way', 'Siding', 'Slope', 'Sound', 'Spur', 'Square', 'Stairs',
        'State Highway', 'Steps', 'Strand', 'Street', 'Strip', 'Subway',
        'Tarn', 'Terrace', 'Thoroughfare', 'Tollway', 'Top', 'Tor', 'Towers',
        'Track', 'Trail', 'Trailer', 'Triangle', 'Trunkway', 'Turn',
        'Underpass', 'Upper', 'Vale', 'Viaduct', 'View', 'Villas', 'Vista',
        'Wade', 'Walk', 'Walkway', 'Way', 'Wynd')

    postcode_formats = (
        # as per https://en.wikipedia.org/wiki/Postcodes_in_Australia
        # NSW
        '1###',
        '20##',
        '21##',
        '22##',
        '23##',
        '24##',
        '25##',
        '2619',
        '262#',
        '263#',
        '264#',
        '265#',
        '266#',
        '267#',
        '268#',
        '269#',
        '27##',
        '28##',
        '292#',
        '293#',
        '294#',
        '295#',
        '296#',
        '297#',
        '298#',
        '299#',
        # ACT
        '02##',
        '260#',
        '261#',
        '290#',
        '291#',
        '2920',
        # VIC
        '3###',
        '8###',
        # QLD
        '4###',
        '9###',
        # SA
        '5###',
        # WA
        '6###',
        # TAS
        '7###',
        # NT
        '08##',
        '09##', )

    states = ('Australian Capital Territory', 'New South Wales',
              'Northern Territory', 'Queensland', 'South Australia',
              'Tasmania', 'Victoria', 'Western Australia')

    states_abbr = ('ACT', 'NSW', 'NT', 'QLD', 'SA', 'TAS', 'VIC', 'WA')

    city_formats = ('{{city_prefix}} {{first_name}}{{city_suffix}}',
                    '{{city_prefix}} {{first_name}}',
                    '{{first_name}}{{city_suffix}}',
                    '{{last_name}}{{city_suffix}}', )

    street_name_formats = ('{{first_name}} {{street_suffix}}',
                           '{{last_name}} {{street_suffix}}')

    street_address_formats = (
        '{{building_number}} {{street_name}}',
        '{{secondary_address}}\n {{building_number}} {{street_name}}', )

    address_formats = (
        "{{street_address}}\n{{city}}, {{state_abbr}}, {{postcode}}", )

    secondary_address_formats = ('Apt. ###', 'Flat ##', 'Suite ###', 'Unit ##',
                                 'Level #', '### /', '## /', '# /')

    def city_prefix(self):
        return self.random_element(self.city_prefixes)

    def secondary_address(self):
        return self.numerify(self.random_element(self.secondary_address_formats))

    def state(self):
        return self.random_element(self.states)

    def state_abbr(self):
        return self.random_element(self.states_abbr)



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
    )

    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
    )

    formats = formats_male + formats_female
    
    # source: http://www.bfs.admin.ch/bfs/portal/fr/index/news/publikationen.html?publicationID=6704
    first_names_male = ['Alain', 'Albert', 'Alexandre', 'André', 'Antonio', 
        'Arthur', 'Bernard', 'Bruno', 'Charles', 'Christian', 'Christophe', 
        'Claude', 'Daniel', 'David', 'Eric', 'Ethan', 'Florian', 'François',
        'Frédéric', 'Gabriel', 'Georges', 'Gilbert', 'Guillaume', 'Gérard', 
        'Henri', 'Hugo', 'Jacques', 'Jean', 'Jean-Claude', 'Jean-Pierre', 
        'Jonathan', 'José', 'Julien', 'Kevin', 'Laurent', 'Louis', 'Loïc', 
        'Luca', 'Lucas', 'Léo', 'Manuel', 'Marcel', 'Mathieu', 'Matteo', 
        'Maurice', 'Maxime', 'Michael', 'Michel', 'Nathan', 'Nicolas', 'Noah',
        'Nolan', 'Olivier', 'Pascal', 'Patrick', 'Paul', 'Philippe', 'Pierre',
        'Raymond', 'René', 'Robert', 'Roger', 'Roland', 'Romain', 'Samuel', 
        'Stéphane', 'Sébastien', 'Thierry', 'Thomas', 'Théo', 'Vincent'
    ]

    first_names_female = ['Alice', 'Alicia', 'Ana', 'Anna', 'Anne', 'Aurélie',
        'Camille', 'Caroline', 'Catherine', 'Chantal', 'Charlotte', 'Chloé', 
        'Christiane', 'Christine', 'Clara', 'Claudine', 'Corinne', 'Céline', 
        'Danielle', 'Denise', 'Eliane', 'Elisa', 'Elisabeth', 'Elodie', 
        'Emilie', 'Emma', 'Eva', 'Fabienne', 'Françoise', 'Georgette', 
        'Germaine', 'Hélène', 'Isabelle', 'Jacqueline', 'Jeanne', 'Jessica',
        'Josiane', 'Julie', 'Laetitia', 'Lara', 'Laura', 'Laurence', 'Liliane',
        'Lisa', 'Lucie', 'Léa', 'Madeleine', 'Manon', 'Marcelle', 'Marguerite',
        'Maria', 'Marianne', 'Marie', 'Mathilde', 'Monique', 'Mélanie', 
        'Nathalie', 'Nelly', 'Nicole', 'Odette', 'Patricia', 'Sandra', 
        'Sandrine', 'Sara', 'Sarah', 'Simone', 'Sophie', 'Stéphanie', 'Suzanne',
        'Sylvie', 'Thérèse', 'Valérie', 'Vanessa', 'Véronique', 'Yvette', 
        'Yvonne', 'Zoé'
    ]
    
    first_names = first_names_male + first_names_female

    # source = http://kunden.eye.ch/swissgen/rsr/index.html
    last_names = ['Aebi', 'Aeby', 'Alber', 'Babey', 'Badan', 'Badel', 'Bahon', 
        'Balmat', 'Barbey', 'Barillon', 'Barman', 'Bavaud', 'Beguin', 
        'Berberat', 'Bernasconi', 'Besançon', 'Besençon', 'Besse', 'Beuchat',
        'Beuret', 'Beurret', 'Blanc', 'Bochud', 'Boechat', 'Boichat', 'Boillat',
        'Bonvin', 'Bonvini', 'Botteron', 'Bourquard', 'Bourquin', 'Bouvier', 
        'Bovet', 'Brahier', 'Brandt', 'Broquet', 'Bugnon', 'Bujard', 'Béguelin',
        'Candaux', 'Carraud', 'Carraux', 'Carron', 'Cattin', 'Chappuis', 
        'Chapuis', 'Charpié', 'Chatriand', 'Chatriant', 'Chaudet', 'Chenaux',
        'Chevalley', 'Chevrolet', 'Chopard', 'Coigny', 'Comman', 'Comment',
        'Comte', 'Conrad', 'Corbat', 'Corboz', 'Cornut', 'Cornuz', 'Corpataux',
        'Cosandey', 'Cosendey', 'Cossy', 'Courvoisier', 'Cousin', 'Cretton', 
        'Crevoisier', 'Crivelli', 'Curdy', 'de Dardel', 'Deladoëy', 'Delèze',
        'Deshusses', 'Diesbach', 'Droz', 'Dubey', 'Duroux', 'Duvanel', 'Délèze',
        'Evéquoz', 'Fonjallaz', 'Francillon', 'Galland', 'Georges', 'Gillièron',
        'Gilliéron', 'Godet', 'Grand', 'Grojean', 'Grosjean', 'Gubéran', 
        'Humbert', 'Isella', 'Jacot-Descombes', 'Jacot-Guillarmod', 'Joly', 
        'Jomini', 'Joye', 'Julliard', 'Maire', 'Marti', 'Martin', 'Marty', 
        'Masseron', 'Matile', 'Mayor', 'Menthonnex', 'Mercier', 'Meyer', 
        'Monnard', 'Monnet', 'Monnet', 'Monney', 'Montandon', 'Morand', 
        'Morard', 'Mottet', 'Mottiez', 'Muriset', 'Musy', 'Müller', 'Niquille',
        'Nusslé', 'Nüsslin', 'Paccot', 'Pachoud', 'Paschoud', 'Pasquier', 
        'Peitrequin', 'Pellet', 'Piccand', 'Polla', 'Privet', 'Quartier', 
        'Rapin', 'Rappaz', 'Rapraz', 'Rey', 'Robadey', 'Robert', 'Romanens', 
        'Rosselat', 'Rosselet', 'Rossellat', 'Sandoz', 'Sansonnens', 'Saudan',
        'Thorens', 'Théraulaz', 'Tinguely', 'Treboux', 'Uldry', 'Vallélian',
        'Vermeil', 'Vienne', 'Vonlanthen', 'Vuille', 'Wicht'
    ] 



# coding=utf-8

from .. import BaseProvider
import itertools


class Provider(BaseProvider):
    """
    This provider is a collection of functions to generate personal profiles and identities.

    """

    def simple_profile(self, sex=None):
        """
        Generates a basic profile with personal informations
        """
        SEX = ["F", "M"]
        if sex not in SEX:
            sex = self.random_element(SEX)
        if sex == 'F':
            name = self.generator.name_female()
        elif sex == 'M':
            name = self.generator.name_male()
        return {
            "username": self.generator.user_name(),
            "name": name,
            "sex": sex,
            "address": self.generator.address(),
            "mail": self.generator.free_email(),

            #"password":self.generator.password()
            "birthdate": self.generator.date(),

        }

    def profile(self, fields=None, sex=None):
        """
        Generates a complete profile.
        If "fields" is not empty, only the fields in the list will be returned
        """
        if fields is None:
            fields = []

        d = {
            "job": self.generator.job(),
            "company": self.generator.company(),
            "ssn": self.generator.ssn(),
            "residence": self.generator.address(),
            "current_location": (self.generator.latitude(), self.generator.longitude()),
            "blood_group": "".join(self.random_element(list(itertools.product(["A", "B", "AB", "0"], ["+", "-"])))),
            "website": [self.generator.url() for i in range(1, self.random_int(2, 5))]
        }

        d = dict(d, **self.generator.simple_profile(sex))
        #field selection
        if len(fields) > 0:
            d = dict((k, v) for (k, v) in d.items() if k in fields)

        return d



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as AddressProvider


class Provider(AddressProvider):

    city_formats = ('{{city_name}}', )

    street_name_formats = ('{{street_name}}', )
    street_address_formats = ('{{street_name}} {{building_number}}', )
    address_formats = ('{{street_address}}\n{{postcode}} {{city}}', )

    building_number_formats = ('###', '##', '#', '#a', '#b', '#c',
        '#a/#', '#b/#', '#c/#')

    postcode_formats = ('#####', )

    street_suffixes_long = (
        '', 'ulica', 'cesta', 'put', 'avenija',
    )
    street_suffixes_short = (
        '', 'ul.', 'c.', 'a.',
    )

    cities = (
        "Bakar", "Beli Manastir", "Belišće", "Benkovac", "Biograd na Moru",
        "Bjelovar", "Buje", "Buzet", "Cres", "Crikvenica", "Čabar", "Čakovec",
        "Čazma", "Daruvar", "Delnice", "Donja Stubica", "Donji Miholjac",
        "Drniš", "Dubrovnik", "Duga Resa", "Dugo Selo", "Đakovo", "Đurđevac",
        "Garešnica", "Glina", "Gospić", "Grubišno Polje",
        "Hrvatska Kostajnica", "Hvar", "Ilok", "Imotski", "Ivanec",
        "Ivanić-Grad", "Jastrebarsko", "Karlovac", "Kastav", "Kaštela",
        "Klanjec", "Knin", "Komiža", "Koprivnica", "Korčula", "Kraljevica",
        "Krapina", "Križevci", "Krk", "Kutina", "Kutjevo", "Labin",
        "Lepoglava", "Lipik", "Ludbreg", "Makarska", "Mali Lošinj",
        "Metković", "Mursko Središće", "Našice", "Nin", "Nova Gradiška",
        "Novalja", "Novi Marof", "Novi Vinodolski", "Novigrad", "Novska",
        "Obrovac", "Ogulin", "Omiš", "Opatija", "Opuzen", "Orahovica",
        "Oroslavje", "Osijek", "Otočac", "Otok", "Ozalj", "Pag", "Pakrac",
        "Pazin", "Petrinja", "Pleternica", "Ploče", "Popovača", "Poreč",
        "Požega", "Pregrada", "Prelog", "Pula", "Rab", "Rijeka", "Rovinj",
        "Samobor", "Senj", "Sinj", "Sisak", "Skradin", "Slatina",
        "Slavonski Brod", "Slunj", "Solin", "Split", "Stari Grad",
        "Supetar", "Sveta Nedelja", "Sveti Ivan Zelina", "Šibenik",
        "Trilj", "Trogir", "Umag", "Valpovo", "Varaždin",
        "Varaždinske Toplice", "Velika Gorica", "Vinkovci", "Virovitica",
        "Vis", "Vodice", "Vodnjan", "Vrbovec", "Vrbovsko", "Vrgorac",
        "Vrlika", "Vukovar", "Zabok", "Zadar", "Zagreb", "Zaprešić", "Zlatar",
    )

    streets = (
        "Arnoldova", "Bakačeva", "Bijenička", "Bosanska", "Bučarova",
        "Cmrok", "Čačkovićeva", "Davor", "Demetrova",
        "Dolac", "Donje Prekrižje", "Draškovićeva", "Dubravkin",
        "Dverce", "Dvoranski prečac", "Glogovac", "Golubovac", "Goljačke",
        "Goljak", "Gornje Prekrižje", "Gračanska", "Gradec", "Grič",
        "Gupčeva zvijezda", "Harmica", "Hercegovačka", "Horvatovac",
        "Ilica", "Istarska", "Jabukovac", "Jadranska", "Jagodnjak",
        "Javorovac", "Jezuitski trg", "Jurišićeva", "Jurjeve",
        "Jurjevska", "Jurkovićeva", "Kamaufova", "Kamenita", "Kamenjak",
        "Kaptol", "Kapucinske", "Klanac Grgura Tepečića", "Klenovac",
        "Klesarski put", "Kozarčev vijenac", "Kožarska", "Kraljevec",
        "Kraljevec II.", "Kraljevečki odvojak", "Kraljevečki ogranak",
        "Krležin gvozd", "Krvavi most", "Ksaver", "Ksaverska", "Kurelčeva",
        "Lisinskoga", "Lobmayerove", "Ljubinkovac", "Magdićeve", "Mala",
        "Male", "Mašekova", "Medvedgradska", "Medveščak", "Mesnička",
        "Mihaljevac", "Mirogojska", "Mletačka", "Mlinarska", "Mlinovi",
        "Mlinske", "Naumovac", "Nemetova", "Nova Ves",
        "Novi Goljak", "Opatička", "Opatovina", "Orlovac",
        "Palmotićeva", "Pantovčak", "Paunovac",
        "Perivoj biskupa Stjepana II.", "Perivoj srpanjskih žrtava",
        "Petrova", "Pod zidom", "Podgaj", "Radnički dol", "Remetska",
        "Ribnjak", "Rikardove", "Rockefellerova", "Rokov perivoj", "Rokova",
        "Ružičnjak", "Skalinska", "Slavujevac", "Splavnica",
        "Srebrnjak", "Streljačka", "Strossmayerovo šetalište", "Svibovac",
        "Svibovac", "Šalata", "Šestinski vijenac", "Šestinski vrh",
        "Šilobodov put", "Šumski prečac", "Tkalčićeva", "Tošovac",
        "Tuškanac", "Vijenac", "Vinogradska", "Visoka", "Višnjica",
        "Višnjičke", "Vitezovićeva", "Vlaška", "Voćarska", "Voćarsko naselje",
        "Vončinina", "Vrazovo šetalište", "Wickerhauserova", "Zamenhofova",
        "Zamenhofove", "Zavojna", "Zelengaj", "Zeleni dol",
        "Zelenjak", "Zmajevac", "Zvonarnička",
    )

    states = (
        "Zagrebačka",
        "Krapinsko-zagorska",
        "Sisačko-moslavačka",
        "Karlovačka",
        "Varaždinska",
        "Koprivničko-križevačka",
        "Bjelovarsko-bilogorska",
        "Primorsko-goranska",
        "Ličko-senjska",
        "Virovitičko-podravska",
        "Požeško-slavonska",
        "Brodsko-posavska",
        "Zadarska",
        "Osječko-baranjska",
        "Šibensko-kninska",
        "Vukovarsko-srijemska",
        "Splitsko-dalmatinska",
        "Istarska",
        "Dubrovačko-neretvanska",
        "Međimurska",
        "Grad Zagreb",
    )

    countries = (
        "Afganistan", "Alandski otoci", "Albanija", "Alžir", "Američka Samoa",
        "Američki Djevičanski Otoci", "Andora", "Angola", "Anguila",
        "Antarktik", "Antigua i Barbuda", "Argentina", "Armenija", "Aruba",
        "Australija", "Austrija", "Azerbajdžan", "Bahami",
        "Bahrein", "Bangladeš", "Barbados", "Belgija", "Belize",
        "Benin", "Bermuda", "Bjelorusija", "Bocvana", "Bolivija",
        "Bosna i Hercegovina", "Božićni Otok", "Brazil",
        "Britanski Djevičanski Otoci", "Britanski Teritorij Indijskog Oceana",
        "Brunei Darussalam", "Bugarska", "Burkina Faso", "Burundi", "Butan",
        "Cipar", "Crna Gora", "Curacao", "Čad", "Čile", "Danska", "Dominika",
        "Dominikanska Republika", "Džibuti", "Egipat", "Ekvador",
        "Ekvatorska Gvineja", "El Salvador", "Eritreja", "Estonija",
        "Etiopija", "Falklandi", "Farski Otoci", "Fidži", "Filipini", "Finska",
        "Francuska", "Francuska Gvajana", "Francuska Polinezija",
        "Francuski Južni Teritoriji", "Gabon", "Gambija", "Gana", "Gibraltar",
        "Vatikan", "Grčka", "Grenada", "Grenland", "Gruzija", "Guadeloupe",
        "Guam", "Guernsey", "Gvajana", "Gvatemala", "Gvineja", "Gvineja Bisau",
        "Haiti", "Honduras", "Hong Kong", "Hrvatska", "Indija", "Indonezija",
        "Irak", "Iran, Islamska Republika", "Irska", "Island", "Isle Of Man",
        "Istočni Timor", "Italija", "Izrael", "Jamajka", "Japan", "Jemen",
        "Jersey", "Jordan", "Južna Afrika",
        "Južna Gruzija i Južni Sendvič Otoci", "Kajmanski Otoci", "Kambodža",
        "Kamerun", "Kanada", "Katar", "Kazakstan", "Kenija", "Kina",
        "Kirgistan", "Kiribati", "Kokosovi Otoci", "Kolumbija", "Komori",
        "Kongo", "Kongo, Demokratska Republika", "Koreja, Južna",
        "Koreja, Sjeverna", "Kosovo", "Kostarika", "Kuba", "Kukovi Otoci",
        "Kuvajt", "Laoska Narodna Demokratska Republika", "Latvija", "Lesoto",
        "Libanon", "Liberija", "Libijska Arapska Džamahirija", "Lihtenštajn",
        "Litva", "Luksemburg", "Madagaskar", "Mađarska", "Majote", "Makao",
        "Makedonija", "Malavi", "Maldivi Maldives", "Malezija", "Mali",
        "Malta", "Maroko", "Maršalovi Otoci", "Martinik", "Mauricijus",
        "Mauritanija", "Meksiko", "Mijanmar", "Mikronezija",
        "Moldavija, Republika", "Monako", "Mongolija", "Montserat", "Mozambik",
        "Namibija", "Nauru", "Nepal", "Niger", "Nigerija", "Nikaragva", "Niue",
        "Nizozemska", "Norveška", "Nova Kaledonija", "Novi Zeland", "Njemačka",
        "Obala Slonovače", "Oman", "Otok Bouvet",
        "Otok Heard i Otoci McDonald", "Otok Norfolk", "Pakistan", "Palau",
        "Palestinsko Područje", "Panama", "Papua Nova Gvineja", "Paragvaj",
        "Peru", "Pitcairn", "Poljska Poland", "Portoriko", "Portugal",
        "Republika Češka", "Reunion", "Ruanda", "Rumunjska", "Rusija",
        "Salamunovi Otoci", "Samoa", "San Marino", "São Tomé ai Príncipe",
        "Saudijska Arabija", "Sejšeli", "Senegal", "Sijera Leone", "Singapur",
        "Sint Maarten", "Sirija", "Sjedinjene Američke Države",
        "Sjeverni Marijanski Otoci", "Slovačka", "Slovenija", "Somalija",
        "Južni Sudan", "Srbija", "Srednjoafrička Republika", "Sudan",
        "Surinam", "Svalbard i Jan Mayen", "Svaziland", "Sveta Helena",
        "Sveti Bartolomej", "Sveti Martin", "Sveti Petar i Miguel",
        "Sv. Kristofor i Nevis", "Sv. Lucija", "Sv. Vincent i Grenadini",
        "Španjolska", "Šri Lanka", "Švedska", "Švicarska", "Tadžikistan",
        "Tajland", "Tajvan", "Tanzanija", "Togo", "Tokelau", "Tonga",
        "Trinidad i Tobago", "Tunis", "Turkmenistan", "Turkski i Kaikos Otoci",
        "Turska", "Tuvalu", "Uganda",
        "Ujedinjene Države Manjih Pacifičkih Otoka",
        "Ujedinjeni Arapski Emirati", "Ukrajina", "Urugvaj", "Uzbekistan",
        "Vanuatu", "Velika Britanija", "Venezuela", "Vijetnam",
        "Wallis i Futuna", "Zambija", "Zapadna Sahara", "Zeleni Rt",
    )

    def city_name(self):
        return self.random_element(self.cities)

    def street_name(self):
        return self.random_element(self.streets)

    def state(self):
        return self.random_element(self.states)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_formats = ("{{city_name}}", )
    building_number_formats = ("%號", "%#號", "%##號", )
    postcode_formats = ("%####", "%##", )
    section_formats = ( "", "", "", "", "%段", )
    street_address_formats = ( "{{street_name}}{{street_name_suffix}}{{section_number}}{{building_number}}", )
    city_formats = ( "{{city_name}}{{city_name_suffix}}", )
    address_formats = ("{{postcode}} {{city}}{{street_address}}{{secondary_address}}", )
    secondary_address_formats = ('#樓', '之#')

    street_names = ("中正", "中山", "民生", "中華", "和平",
                    "中興", "仁愛", "復興", "民族", "民權",
                    "忠孝", "信義", "成功", "新興", "新生",
                    "動物園", "淡水", "新生", "文化", "大同",
                    "三民", "新生", "光復", "自強", "光明",
                    "公園", "文山", "松山", "新店", "建國",
                    "西門", "古亭", "迴龍", "中山", "新莊",
                    "蘆洲", "永安", "四維", "大橋頭", "府中",
                    "福德", "大同", "文昌", "土城", "博愛",
                    "象山", "光華", "太平", "水源", "莒光",
                    "廣慈", "大仁", "中央", "大智", "林森",
                    "長春", "南", "劍南", "大坪", "國凱"
                    "八德", "天母", "東興", "勝利", "頂福州",
                    "東湖", "大勇", "民有", "自由", "長安",
                    "明德", "大安", "龍山寺", "德", "忠義",
                    "中和", "自由", "新埔", "永和", "延平",
                    "正義", "五福", "華興", "育英", "平和",
                    "福安", "小碧潭", "永寧", "育英", "興",
                    "自立", "民享", "昆陽", "民治", "關渡",
                    "學府", "奇岩", "紅樹林", "和街", "民富",
                    "關渡", "北投", "石牌", "芝山", "景美",
                    "士林", "劍潭", "雙連", "新北投", "萬隆")

    street_suffixes = ( "路", "街", "巷" )

    cities = ("基隆", "台北", "新北", "桃園", "新竹",
              "新竹", "苗栗", "台中", "彰化", "南投",
              "雲林", "嘉義", "桃園", "台南", "高雄",
              "屏東", "台東", "花蓮", "宜蘭", "澎湖",
              "金門", "連江", "太保", "朴子", "馬公",
              "頭份", "臺東", "斗六", "員林", "竹北",
              "平鎮", "臺中", "八德", "板橋", "大里",
              "鳳山", "豐原", "蘆洲", "蘆竹", "三重",
              "樹林", "太平", "新營", "新營", "汐止",
              "楊梅", "永和", "永康", "中和", "中壢",
              "阿里山", "白沙", "褒忠", "北斗", "北竿",
              "北港", "卑南", "草屯", "梅山", "牡丹",
              "橫山", "光復", "關山", "古坑", "竹田")

    city_suffixes = ( "市", "縣" )

    # from
    countries = ("阿爾巴尼亞", "剛果共和國", "阿爾及利亞", "丹麥",
                 "安哥拉", "多明尼加", "安圭拉", "多米尼克",
                 "阿根廷", "厄瓜多爾", "亞美尼亞", "埃及",
                 "阿路巴", "薩爾瓦多", "澳大利亞", "厄利垂亞",
                 "奧地利", "愛沙尼亞", "亞塞拜然", "衣索匹亞",
                 "巴哈馬", "斐濟", "巴林", "芬蘭", "孟加拉", "法屬玻里尼西亞",
                 "法國", "巴貝多", "加彭", "白俄羅斯", "喬治亞",
                 "比利時", "德國", "貝里斯", "迦納", "貝南", "直布羅陀",
                 "百慕達", "英國", "不丹", "希臘", "玻利維亞", "格瑞那達",
                 "波希尼亞及赫塞哥維那", "瓜地馬拉", "波札那", "幾內亞",
                 "巴西", "蓋亞那", "汶萊", "海地", "保加利亞", "宏都拉斯",
                 "布吉納法索", "香港", "蒲隆地", "匈牙利", "柬埔寨", "冰島",
                 "喀麥隆", "印度", "加拿大", "印尼", "維德角島", "依朗",
                 "開曼群島", "伊拉克", "中非共和國", "愛爾蘭", "查德", "以色列",
                 "智利", "義大利", "中國大陸", "牙買加", "哥倫比亞", "日本",
                 "剛果", "約旦", "科克群島", "肯亞", "哥斯大黎加", "韓國",
                 "象牙海岸", "科威特", "克羅埃西亞", "寮國", "塞浦路斯", "拉脫維亞",
                 "捷克", "賴索托", "盧森堡", "聖露西亞", "澳門", "聖文森及格瑞那丁",
                 "馬其頓", "聖多美及普林西比", "馬達加斯加", "沙烏地阿拉伯",
                 "馬拉威", "塞內加爾", "馬來西亞", "塞席爾", "馬爾地夫", "獅子山",
                 "馬利", "新加坡", "馬爾他", "斯洛伐克", "模里西斯", "斯洛維尼亞",
                 "茅利塔尼亞", "索羅門群島", "墨西哥", "索馬利亞",
                 "摩爾多瓦", "南非", "蒙古", "西班牙", "摩洛哥", "斯里蘭卡",
                 "緬甸", "蘇丹", "納米比亞", "蘇利南", "諾魯", "史瓦濟蘭",
                 "尼泊爾", "瑞典", "荷蘭", "瑞士", "新喀里多尼亞", "敘利亞",
                 "紐西蘭", "坦尚尼亞", "尼日", "泰國", "奈及利亞", "多哥",
                 "挪威", "千里達及托貝哥", "阿曼", "突尼西亞", "巴基斯坦", "土耳其",
                 "巴拿馬", "烏干達", "巴布亞紐幾內亞", "烏克蘭",
                 "巴拉圭", "阿拉伯聯合大公國", "秘魯", "美國", "菲律賓", "烏拉圭",
                 "波蘭", "委內瑞拉", "葡萄牙", "越南", "卡達", "西薩摩亞",
                 "羅馬尼亞", "葉門", "俄羅斯", "尚比亞", "盧安達", "辛巴威",
                 "聖克里斯多福及尼維斯")

    def secondary_address(self):
        return self.numerify(self.random_element(self.secondary_address_formats))

    def building_number(self):
        return self.numerify(self.random_element(self.building_number_formats))

    def street_name(self):
        return self.random_element(self.street_names)

    def street_name_suffix(self):
        return self.random_element(self.street_suffixes)

    def city_name(self):
        return self.random_element(self.cities)

    def city_name_suffix(self):
        return self.random_element(self.city_suffixes)

    def section_number(self):
        return self.numerify(self.random_element(self.section_formats))
